"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toReadonlyArray = exports.intercalate = exports.reduceWithEffect = exports.getReduceRightComposition = exports.getFoldMapComposition = exports.getReduceComposition = void 0;
var function_1 = require("./function");
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `reduce` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var getReduceComposition = function (F, G) {
    return function (b, f) {
        return F.reduce(b, function (b, ga) { return (0, function_1.pipe)(ga, G.reduce(b, f)); });
    };
};
exports.getReduceComposition = getReduceComposition;
/**
 * `foldMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var getFoldMapComposition = function (F, G) {
    return function (M) { return function (f) { return F.foldMap(M)(G.foldMap(M)(f)); }; };
};
exports.getFoldMapComposition = getFoldMapComposition;
/**
 * `reduceRight` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var getReduceRightComposition = function (F, G) {
    return function (b, f) { return F.reduceRight(b, function (ga, b) { return G.reduceRight(b, f)(ga); }); };
};
exports.getReduceRightComposition = getReduceRightComposition;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Similar to 'reduce', but the result is encapsulated in a monad.
 *
 * Note: this function is not generally stack-safe, e.g., for monads which build up thunks a la `IO`.
 *
 * @example
 * import { reduceWithEffect } from 'fp-ts/Foldable'
 * import { Flattenable, some } from 'fp-ts/Option'
 * import { make, Foldable } from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const tree = make(1, [make(2), make(3), make(4)])
 * assert.deepStrictEqual(pipe(tree, reduceWithEffect(Foldable)(Flattenable)(some(0), (b, a) => (a > 2 ? some(b + a) : some(b)))), some(7))
 *
 * @since 3.0.0
 */
function reduceWithEffect(F) {
    return function (M) { return function (mb, f) {
        return F.reduce(mb, function (mb, a) {
            return (0, function_1.pipe)(mb, M.flatMap(function (b) { return f(b, a); }));
        });
    }; };
}
exports.reduceWithEffect = reduceWithEffect;
/**
 * Fold a data structure, accumulating values in some `Monoid`, combining adjacent elements using the specified separator.
 *
 * @example
 * import { intercalate } from 'fp-ts/Foldable'
 * import { Monoid } from 'fp-ts/string'
 * import * as T from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const tree = T.make('a', [T.make('b'), T.make('c'), T.make('d')])
 * assert.strictEqual(pipe(tree, intercalate(T.Foldable)(Monoid)('|')), 'a|b|c|d')
 *
 * @since 3.0.0
 */
function intercalate(F) {
    return function (M) {
        return function (sep) {
            return function (fm) {
                var go = function (_a, m) {
                    var init = _a[0], acc = _a[1];
                    return init ? [false, m] : [false, (0, function_1.pipe)(acc, M.combine(sep), M.combine(m))];
                };
                return (0, function_1.pipe)(fm, F.reduce([true, M.empty], go))[1];
            };
        };
    };
}
exports.intercalate = intercalate;
/**
 * Transforms a `Foldable` into a read-only array.
 *
 * @example
 * import { toReadonlyArray } from 'fp-ts/Foldable'
 * import { Foldable, make } from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const tree = make(1, [make(2), make(3), make(4)])
 * assert.deepStrictEqual(pipe(tree, toReadonlyArray(Foldable)), [1, 2, 3, 4])
 *
 * @since 3.0.0
 */
function toReadonlyArray(F) {
    return function (self) {
        return (0, function_1.pipe)(self, F.reduce([], function (acc, a) {
            acc.push(a);
            return acc;
        }));
    };
}
exports.toReadonlyArray = toReadonlyArray;
