"use strict";
/**
 * A `Magma` is a pair `(A, combine)` in which `A` is a non-empty set and `combine` is a binary operation on `A`
 *
 * See [Semigroup](https://gcanti.github.io/fp-ts/modules/Semigroup.ts.html) for some instances.
 *
 * @since 3.0.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.combineAll = exports.endo = exports.filterSecond = exports.filterFirst = exports.reverse = void 0;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * The dual of a `Magma`, obtained by swapping the arguments of `combine`.
 *
 * @example
 * import { reverse, combineAll } from 'fp-ts/Magma'
 * import * as N from 'fp-ts/number'
 *
 * const subAll = combineAll(reverse(N.MagmaSub))(0)
 *
 * assert.deepStrictEqual(subAll([1, 2, 3]), 2)
 *
 * @category combinators
 * @since 3.0.0
 */
var reverse = function (M) { return ({
    combine: function (that) { return function (self) { return M.combine(self)(that); }; }
}); };
exports.reverse = reverse;
/**
 * @category combinators
 * @since 3.0.0
 */
var filterFirst = function (predicate) {
    return function (M) { return ({
        combine: function (that) { return function (self) { return predicate(self) ? M.combine(that)(self) : that; }; }
    }); };
};
exports.filterFirst = filterFirst;
/**
 * @category combinators
 * @since 3.0.0
 */
var filterSecond = function (predicate) {
    return function (M) { return ({
        combine: function (that) { return function (self) { return predicate(that) ? M.combine(that)(self) : self; }; }
    }); };
};
exports.filterSecond = filterSecond;
/**
 * @category combinators
 * @since 3.0.0
 */
var endo = function (f) {
    return function (M) { return ({
        combine: function (that) { return function (self) { return M.combine(f(that))(f(self)); }; }
    }); };
};
exports.endo = endo;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Given a sequence of `as`, combine them and return the total.
 *
 * If `as` is empty, return the provided `startWith` value.
 *
 * @example
 * import { combineAll } from 'fp-ts/Magma'
 * import * as N from 'fp-ts/number'
 *
 * const subAll = combineAll(N.MagmaSub)(0)
 *
 * assert.deepStrictEqual(subAll([1, 2, 3]), -6)
 *
 * @since 3.0.0
 */
var combineAll = function (M) {
    return function (startWith) {
        return function (as) {
            return as.reduce(function (a, acc) { return M.combine(acc)(a); }, startWith);
        };
    };
};
exports.combineAll = combineAll;
