/**
 * The `Apply` class provides the `ap` which is used to apply a function to an argument under a type constructor.
 *
 * `Apply` can be used to lift functions of two or more arguments to work on values wrapped with the type constructor
 * `f`.
 *
 * Instances must satisfy the following law in addition to the `Functor` laws:
 *
 * 1. Associative composition: `fbc |> map(bc => ab => a => bc(ab(a))) |> ap(fab) <-> fbc |> ap(fab |> ap(fa))`
 *
 * Formally, `Apply` represents a strong lax semi-monoidal endofunctor.
 *
 * @example
 * import * as O from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * const f = (a: string) => (b: number) => (c: boolean) => a + String(b) + String(c)
 * const fa: O.Option<string> = O.some('s')
 * const fb: O.Option<number> = O.some(1)
 * const fc: O.Option<boolean> = O.some(true)
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     // lift a function
 *     O.some(f),
 *     // apply the first argument
 *     O.ap(fa),
 *     // apply the second argument
 *     O.ap(fb),
 *     // apply the third argument
 *     O.ap(fc)
 *   ),
 *   O.some('s1true')
 * )
 *
 * @since 3.0.0
 */
import { flow, pipe } from './function';
import * as semigroup from './Semigroup';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `ap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const getApComposition = (Applyλ, Applyμ) => (fa) => {
    return flow(Applyλ.map((gab) => (ga) => Applyμ.ap(ga)(gab)), Applyλ.ap(fa));
};
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = (Applyλ) => (second) => flow(Applyλ.map((a) => () => a), Applyλ.ap(second));
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = (Applyλ) => (second) => flow(Applyλ.map(() => (b) => b), Applyλ.ap(second));
/**
 * @category combinators
 * @since 3.0.0
 */
export const bindPar = (Applyλ) => (name, fb) => flow(Applyλ.map((a) => (b) => Object.assign({}, a, { [name]: b })), Applyλ.ap(fb));
/**
 * @category combinators
 * @since 3.0.0
 */
export const apT = (Applyλ) => (fb) => (fas) => pipe(fas, Applyλ.map((a) => (b) => [...a, b]), Applyλ.ap(fb));
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Lift a semigroup into 'λ', the inner values are combined using the provided `Semigroup`.
 *
 * @since 3.0.0
 */
export const getApplySemigroup = (Applyλ) => (S) => {
    const f = semigroup.reverse(S).combine;
    return {
        combine: (second) => (first) => pipe(first, Applyλ.map(f), Applyλ.ap(second))
    };
};
