/**
 * @since 3.0.0
 */
import * as apply from './Apply';
import * as either from './Either';
import { flow, pipe } from './function';
import * as functor from './Functor';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const right = (F) => (a) => F.of(either.right(a));
/**
 * @since 3.0.0
 */
export const left = (F) => (e) => F.of(either.left(e));
/**
 * @since 3.0.0
 */
export function rightF(F) {
    return F.map(either.right);
}
/**
 * @since 3.0.0
 */
export function leftF(F) {
    return F.map(either.left);
}
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @since 3.0.0
 */
export const map = (F) => functor.getMapComposition(F, either.Functor);
/**
 * @since 3.0.0
 */
export const ap = (F) => {
    return apply.getApComposition(F, either.Apply);
};
/**
 * @since 3.0.0
 */
export const flatMap = (M) => (f) => (self) => {
    return pipe(self, M.flatMap((e) => (either.isLeft(e) ? M.of(e) : f(e.right))));
};
/**
 * @since 3.0.0
 */
export const combineK = (M) => (second) => (first) => {
    return pipe(first, M.flatMap((e) => either.isLeft(e) ? second() : M.of(e)));
};
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const mapBoth = (F) => {
    return flow(either.mapBoth, F.map);
};
/**
 * @category type class operations
 * @since 3.0.0
 */
export const mapLeft = (F) => (f) => {
    return F.map(either.mapError(f));
};
/**
 * @since 3.0.0
 */
export const getValidatedCombineK = (M, S) => (second) => (first) => {
    const rightM = right(M);
    return pipe(first, M.flatMap(either.match((e1) => pipe(second(), M.map(either.mapError((e2) => S.combine(e2)(e1)))), rightM)));
};
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function match(F) {
    return flow(either.match, F.map);
}
/**
 * @since 3.0.0
 */
export const matchWithEffect = (M) => (onError, onSuccess) => {
    return M.flatMap(either.match(onError, onSuccess));
};
/**
 * @since 3.0.0
 */
export const getOrElse = (F) => (onError) => {
    return F.map(either.getOrElse(onError));
};
/**
 * @since 3.0.0
 */
export const getOrElseE = (M) => (onError) => (self) => {
    return pipe(self, M.flatMap(either.match(onError, M.of)));
};
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const orElse = (M) => (onError) => (self) => {
    return pipe(self, M.flatMap((e) => either.isLeft(e) ? onError(e.left) : M.of(e)));
};
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapLeft = (M) => {
    const orElseM = orElse(M);
    return (onError) => (self) => {
        return pipe(self, orElseM((e) => pipe(onError(e), M.map((eb) => (either.isLeft(eb) ? eb : either.left(e))))));
    };
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export function swap(F) {
    return F.map(either.swap);
}
/**
 * @since 3.0.0
 */
export function toUnion(F) {
    return F.map(either.toUnion);
}
/**
 * @since 3.0.0
 */
export const bracket = (M) => (acquire, use, release) => {
    const leftM = left(M);
    return pipe(acquire, M.flatMap(either.match(leftM, (a) => pipe(use(a), M.flatMap((e) => pipe(release(a, e), M.flatMap(either.match(leftM, () => M.of(e)))))))));
};
