import { pipe } from './function';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `reduce` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const getReduceComposition = (F, G) => (b, f) => F.reduce(b, (b, ga) => pipe(ga, G.reduce(b, f)));
/**
 * `foldMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const getFoldMapComposition = (F, G) => {
    return (M) => (f) => F.foldMap(M)(G.foldMap(M)(f));
};
/**
 * `reduceRight` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const getReduceRightComposition = (F, G) => {
    return (b, f) => F.reduceRight(b, (ga, b) => G.reduceRight(b, f)(ga));
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Similar to 'reduce', but the result is encapsulated in a monad.
 *
 * Note: this function is not generally stack-safe, e.g., for monads which build up thunks a la `IO`.
 *
 * @example
 * import { reduceWithEffect } from 'fp-ts/Foldable'
 * import { Flattenable, some } from 'fp-ts/Option'
 * import { make, Foldable } from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const tree = make(1, [make(2), make(3), make(4)])
 * assert.deepStrictEqual(pipe(tree, reduceWithEffect(Foldable)(Flattenable)(some(0), (b, a) => (a > 2 ? some(b + a) : some(b)))), some(7))
 *
 * @since 3.0.0
 */
export function reduceWithEffect(F) {
    return (M) => (mb, f) => F.reduce(mb, (mb, a) => pipe(mb, M.flatMap((b) => f(b, a))));
}
/**
 * Fold a data structure, accumulating values in some `Monoid`, combining adjacent elements using the specified separator.
 *
 * @example
 * import { intercalate } from 'fp-ts/Foldable'
 * import { Monoid } from 'fp-ts/string'
 * import * as T from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const tree = T.make('a', [T.make('b'), T.make('c'), T.make('d')])
 * assert.strictEqual(pipe(tree, intercalate(T.Foldable)(Monoid)('|')), 'a|b|c|d')
 *
 * @since 3.0.0
 */
export function intercalate(F) {
    return (M) => (sep) => (fm) => {
        const go = ([init, acc], m) => init ? [false, m] : [false, pipe(acc, M.combine(sep), M.combine(m))];
        return pipe(fm, F.reduce([true, M.empty], go))[1];
    };
}
/**
 * Transforms a `Foldable` into a read-only array.
 *
 * @example
 * import { toReadonlyArray } from 'fp-ts/Foldable'
 * import { Foldable, make } from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const tree = make(1, [make(2), make(3), make(4)])
 * assert.deepStrictEqual(pipe(tree, toReadonlyArray(Foldable)), [1, 2, 3, 4])
 *
 * @since 3.0.0
 */
export function toReadonlyArray(F) {
    return (self) => pipe(self, F.reduce([], (acc, a) => {
        acc.push(a);
        return acc;
    }));
}
