/**
 * A `Magma` is a pair `(A, combine)` in which `A` is a non-empty set and `combine` is a binary operation on `A`
 *
 * See [Semigroup](https://gcanti.github.io/fp-ts/modules/Semigroup.ts.html) for some instances.
 *
 * @since 3.0.0
 */
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * The dual of a `Magma`, obtained by swapping the arguments of `combine`.
 *
 * @example
 * import { reverse, combineAll } from 'fp-ts/Magma'
 * import * as N from 'fp-ts/number'
 *
 * const subAll = combineAll(reverse(N.MagmaSub))(0)
 *
 * assert.deepStrictEqual(subAll([1, 2, 3]), 2)
 *
 * @category combinators
 * @since 3.0.0
 */
export const reverse = (M) => ({
    combine: (that) => (self) => M.combine(self)(that)
});
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterFirst = (predicate) => (M) => ({
    combine: (that) => (self) => predicate(self) ? M.combine(that)(self) : that
});
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterSecond = (predicate) => (M) => ({
    combine: (that) => (self) => predicate(that) ? M.combine(that)(self) : self
});
/**
 * @category combinators
 * @since 3.0.0
 */
export const endo = (f) => (M) => ({
    combine: (that) => (self) => M.combine(f(that))(f(self))
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Given a sequence of `as`, combine them and return the total.
 *
 * If `as` is empty, return the provided `startWith` value.
 *
 * @example
 * import { combineAll } from 'fp-ts/Magma'
 * import * as N from 'fp-ts/number'
 *
 * const subAll = combineAll(N.MagmaSub)(0)
 *
 * assert.deepStrictEqual(subAll([1, 2, 3]), -6)
 *
 * @since 3.0.0
 */
export const combineAll = (M) => (startWith) => (as) => as.reduce((a, acc) => M.combine(acc)(a), startWith);
