import * as monoidK from './MonoidK';
import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as eq from './Eq';
import * as filterable from './Filterable';
import * as filterableWithIndex from './FilterableWithIndex';
import * as fromOption_ from './FromOption';
import * as fromEither_ from './FromEither';
import { identity, pipe } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as number from './number';
import * as ord from './Ord';
import * as readonlyNonEmptyArray from './ReadonlyNonEmptyArray';
import * as traversable from './Traversable';
import * as filterableWithEffect from './FilterableWithEffect';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Return a `ReadonlyArray` of length `n` with element `i` initialized with `f(i)`.
 *
 * **Note**. `n` is normalized to a non negative integer.
 *
 * @example
 * import { makeBy } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const double = (n: number): number => n * 2
 * assert.deepStrictEqual(pipe(5, makeBy(double)), [0, 2, 4, 6, 8])
 *
 * @category constructors
 * @since 3.0.0
 */
export const makeBy = (f) => (n) => n <= 0 ? empty : readonlyNonEmptyArray.makeBy(f)(n);
/**
 * Create a `ReadonlyArray` containing a value repeated the specified number of times.
 *
 * **Note**. `n` is normalized to a non negative integer.
 *
 * @example
 * import { replicate } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(3, replicate('a')), ['a', 'a', 'a'])
 *
 * @category constructors
 * @since 3.0.0
 */
export const replicate = (a) => makeBy(() => a);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = (ma) => (_.isNone(ma) ? empty : [ma.value]);
/**
 * Transforms an `Either` to a `ReadonlyArray`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = (e) => (_.isLeft(e) ? empty : [e.right]);
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = (onEmpty, onNonEmpty) => (as) => isNonEmpty(as) ? onNonEmpty(as) : onEmpty();
/**
 * Break a `ReadonlyArray` into its first element and remaining elements.
 *
 * @example
 * import { matchLeft } from 'fp-ts/ReadonlyArray'
 *
 * const len: <A>(as: ReadonlyArray<A>) => number = matchLeft(() => 0, (_, tail) => 1 + len(tail))
 * assert.strictEqual(len([1, 2, 3]), 3)
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchLeft = (onEmpty, onNonEmpty) => (as) => isNonEmpty(as) ? onNonEmpty(readonlyNonEmptyArray.head(as), readonlyNonEmptyArray.tail(as)) : onEmpty();
/**
 * Break a `ReadonlyArray` into its initial elements and the last element.
 *
 * @category destructors
 * @since 3.0.0
 */
export const matchRight = (onEmpty, onNonEmpty) => (as) => isNonEmpty(as) ? onNonEmpty(readonlyNonEmptyArray.init(as), readonlyNonEmptyArray.last(as)) : onEmpty();
export function comprehension(input, f, g = () => true) {
    const go = (as, input) => isNonEmpty(input)
        ? pipe(readonlyNonEmptyArray.head(input), flatMap((head) => go(append(head)(as), readonlyNonEmptyArray.tail(input))))
        : g(...as)
            ? [f(...as)]
            : empty;
    return go(empty, input);
}
/**
 * @category combinators
 * @since 3.0.0
 */
export const concat = (second) => (self) => isEmpty(self) ? second : isEmpty(second) ? self : self.concat(second);
/**
 * Fold a `ReadonlyArray` from the left, keeping all intermediate results instead of only the final result.
 *
 * @example
 * import { scanLeft } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(scanLeft(10, (b, a: number) => b - a)([1, 2, 3]), [10, 9, 7, 4])
 *
 * @category combinators
 * @since 3.0.0
 */
export const scanLeft = (b, f) => (as) => {
    const len = as.length;
    const out = new Array(len + 1);
    out[0] = b;
    for (let i = 0; i < len; i++) {
        out[i + 1] = f(out[i], as[i]);
    }
    return out;
};
/**
 * Fold a `ReadonlyArray` from the right, keeping all intermediate results instead of only the final result.
 *
 * @example
 * import { scanRight } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(scanRight(10, (a: number, b) => b - a)([1, 2, 3]), [4, 5, 7, 10])
 *
 * @category combinators
 * @since 3.0.0
 */
export const scanRight = (b, f) => (as) => {
    const len = as.length;
    const out = new Array(len + 1);
    out[len] = b;
    for (let i = len - 1; i >= 0; i--) {
        out[i] = f(as[i], out[i + 1]);
    }
    return out;
};
/**
 * Test whether a `ReadonlyArray` is empty.
 *
 * @example
 * import { isEmpty } from 'fp-ts/ReadonlyArray'
 *
 * assert.strictEqual(isEmpty([]), true)
 *
 * @since 3.0.0
 */
export const isEmpty = (as) => as.length === 0;
/**
 * Test whether a `ReadonlyArray` is non empty narrowing down the type to `NonEmptyReadonlyArray<A>`
 *
 * @category guards
 * @since 3.0.0
 */
export const isNonEmpty = _.isNonEmpty;
/**
 * Calculate the number of elements in a `ReadonlyArray`.
 *
 * @since 3.0.0
 */
export const size = (as) => as.length;
/**
 * Test whether a `ReadonlyArray` contains a particular index
 *
 * @since 3.0.0
 */
export const isOutOfBound = readonlyNonEmptyArray.isOutOfBound;
/**
 * This function provides a safe way to read a value at a particular index from a `ReadonlyArray`
 *
 * @example
 * import { lookup } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3], lookup(1)), some(2))
 * assert.deepStrictEqual(pipe([1, 2, 3], lookup(3)), none)
 *
 * @since 3.0.0
 */
export const lookup = (i) => (as) => isOutOfBound(i, as) ? _.none : _.some(as[i]);
/**
 * Prepend an element to the front of a `ReadonlyArray`, creating a new `ReadonlyNonEmptyArray`.
 *
 * @example
 * import { prepend } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3], prepend(0)), [0, 1, 2, 3])
 *
 * @category constructors
 * @since 3.0.0
 */
export const prepend = readonlyNonEmptyArray.prepend;
/**
 * Append an element to the end of a `ReadonlyArray`, creating a new `ReadonlyNonEmptyArray`.
 *
 * @example
 * import { append } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3], append(4)), [1, 2, 3, 4])
 *
 * @category constructors
 * @since 3.0.0
 */
export const append = readonlyNonEmptyArray.append;
/**
 * Get the first element of a `ReadonlyArray`, or `None` if the `ReadonlyArray` is empty.
 *
 * @example
 * import { head } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(head([1, 2, 3]), some(1))
 * assert.deepStrictEqual(head([]), none)
 *
 * @since 3.0.0
 */
export const head = (as) => isNonEmpty(as) ? _.some(readonlyNonEmptyArray.head(as)) : _.none;
/**
 * Get the last element in a `ReadonlyArray`, or `None` if the `ReadonlyArray` is empty.
 *
 * @example
 * import { last } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(last([1, 2, 3]), some(3))
 * assert.deepStrictEqual(last([]), none)
 *
 * @since 3.0.0
 */
export const last = (as) => isNonEmpty(as) ? _.some(readonlyNonEmptyArray.last(as)) : _.none;
/**
 * Get all but the first element of a `ReadonlyArray`, creating a new `ReadonlyArray`, or `None` if the `ReadonlyArray` is empty.
 *
 * @example
 * import { tail } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(tail([1, 2, 3]), some([2, 3]))
 * assert.deepStrictEqual(tail([]), none)
 *
 * @since 3.0.0
 */
export const tail = (as) => isNonEmpty(as) ? _.some(readonlyNonEmptyArray.tail(as)) : _.none;
/**
 * Get all but the last element of a `ReadonlyArray`, creating a new `ReadonlyArray`, or `None` if the `ReadonlyArray` is empty.
 *
 * @example
 * import { init } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(init([1, 2, 3]), some([1, 2]))
 * assert.deepStrictEqual(init([]), none)
 *
 * @since 3.0.0
 */
export const init = (as) => isNonEmpty(as) ? _.some(readonlyNonEmptyArray.init(as)) : _.none;
/**
 * Keep only a max number of elements from the start of an `ReadonlyArray`, creating a new `ReadonlyArray`.
 *
 * **Note**. `n` is normalized to a non negative integer.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const input: ReadonlyArray<number> = [1, 2, 3]
 * assert.deepStrictEqual(pipe(input, RA.takeLeft(2)), [1, 2])
 *
 * // out of bounds
 * assert.strictEqual(pipe(input, RA.takeLeft(4)), input)
 * assert.strictEqual(pipe(input, RA.takeLeft(-1)), input)
 *
 * @category combinators
 * @since 3.0.0
 */
export const takeLeft = (n) => (as) => isOutOfBound(n, as) ? as : n === 0 ? empty : as.slice(0, n);
/**
 * Keep only a max number of elements from the end of an `ReadonlyArray`, creating a new `ReadonlyArray`.
 *
 * **Note**. `n` is normalized to a non negative integer.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const input: ReadonlyArray<number> = [1, 2, 3]
 * assert.deepStrictEqual(pipe(input, RA.takeRight(2)), [2, 3])
 *
 * // out of bounds
 * assert.strictEqual(pipe(input, RA.takeRight(4)), input)
 * assert.strictEqual(pipe(input, RA.takeRight(-1)), input)
 *
 * @since 3.0.0
 */
export const takeRight = (n) => (as) => isOutOfBound(n, as) ? as : n === 0 ? empty : as.slice(-n);
export function takeLeftWhile(predicate) {
    return (as) => {
        const out = [];
        for (const a of as) {
            if (!predicate(a)) {
                break;
            }
            out.push(a);
        }
        const len = out.length;
        return len === as.length ? as : len === 0 ? empty : out;
    };
}
const spanLeftIndex = (as, predicate) => {
    const l = as.length;
    let i = 0;
    for (; i < l; i++) {
        if (!predicate(as[i])) {
            break;
        }
    }
    return i;
};
export function spanLeft(predicate) {
    return (as) => splitAt(spanLeftIndex(as, predicate))(as);
}
/**
 * Drop a max number of elements from the start of an `ReadonlyArray`, creating a new `ReadonlyArray`.
 *
 * **Note**. `n` is normalized to a non negative integer.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const input: ReadonlyArray<number> = [1, 2, 3]
 * assert.deepStrictEqual(pipe(input, RA.dropLeft(2)), [3])
 * assert.strictEqual(pipe(input, RA.dropLeft(0)), input)
 * assert.strictEqual(pipe(input, RA.dropLeft(-1)), input)
 *
 * @category combinators
 * @since 3.0.0
 */
export const dropLeft = (n) => (as) => n <= 0 || isEmpty(as) ? as : n >= as.length ? empty : as.slice(n, as.length);
/**
 * Drop a max number of elements from the end of an `ReadonlyArray`, creating a new `ReadonlyArray`.
 *
 * **Note**. `n` is normalized to a non negative integer.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const input: ReadonlyArray<number> = [1, 2, 3]
 * assert.deepStrictEqual(pipe(input, RA.dropRight(2)), [1])
 * assert.strictEqual(pipe(input, RA.dropRight(0)), input)
 * assert.strictEqual(pipe(input, RA.dropRight(-1)), input)
 *
 * @category combinators
 * @since 3.0.0
 */
export const dropRight = (n) => (as) => n <= 0 || isEmpty(as) ? as : n >= as.length ? empty : as.slice(0, as.length - n);
export function dropLeftWhile(predicate) {
    return (as) => {
        const i = spanLeftIndex(as, predicate);
        return i === 0 ? as : i === as.length ? empty : as.slice(i);
    };
}
/**
 * Find the first index for which a predicate holds
 *
 * @example
 * import { findIndex } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(findIndex((n: number) => n === 2)([1, 2, 3]), some(1))
 * assert.deepStrictEqual(findIndex((n: number) => n === 2)([]), none)
 *
 * @since 3.0.0
 */
export const findIndex = (predicate) => (as) => {
    const len = as.length;
    for (let i = 0; i < len; i++) {
        if (predicate(as[i])) {
            return _.some(i);
        }
    }
    return _.none;
};
export function findFirst(predicate) {
    return (as) => {
        const len = as.length;
        for (let i = 0; i < len; i++) {
            if (predicate(as[i])) {
                return _.some(as[i]);
            }
        }
        return _.none;
    };
}
/**
 * Find the first element returned by an option based selector function
 *
 * @example
 * import { findFirstMap } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * interface Person {
 *   name: string
 *   age?: number
 * }
 *
 * const persons: ReadonlyArray<Person> = [{ name: 'John' }, { name: 'Mary', age: 45 }, { name: 'Joey', age: 28 }]
 *
 * // returns the name of the first person that has an age
 * assert.deepStrictEqual(findFirstMap((p: Person) => (p.age === undefined ? none : some(p.name)))(persons), some('Mary'))
 *
 * @since 3.0.0
 */
export const findFirstMap = (f) => (as) => {
    const len = as.length;
    for (let i = 0; i < len; i++) {
        const v = f(as[i]);
        if (_.isSome(v)) {
            return v;
        }
    }
    return _.none;
};
export function findLast(predicate) {
    return (as) => {
        const len = as.length;
        for (let i = len - 1; i >= 0; i--) {
            if (predicate(as[i])) {
                return _.some(as[i]);
            }
        }
        return _.none;
    };
}
/**
 * Find the last element returned by an option based selector function
 *
 * @example
 * import { findLastMap } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * interface Person {
 *   name: string
 *   age?: number
 * }
 *
 * const persons: ReadonlyArray<Person> = [{ name: 'John' }, { name: 'Mary', age: 45 }, { name: 'Joey', age: 28 }]
 *
 * // returns the name of the last person that has an age
 * assert.deepStrictEqual(findLastMap((p: Person) => (p.age === undefined ? none : some(p.name)))(persons), some('Joey'))
 *
 * @since 3.0.0
 */
export const findLastMap = (f) => (as) => {
    const len = as.length;
    for (let i = len - 1; i >= 0; i--) {
        const v = f(as[i]);
        if (_.isSome(v)) {
            return v;
        }
    }
    return _.none;
};
/**
 * Returns the index of the last element of the list which matches the predicate
 *
 * @example
 * import { findLastIndex } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * interface X {
 *   a: number
 *   b: number
 * }
 * const xs: ReadonlyArray<X> = [{ a: 1, b: 0 }, { a: 1, b: 1 }]
 * assert.deepStrictEqual(findLastIndex((x: { a: number }) => x.a === 1)(xs), some(1))
 * assert.deepStrictEqual(findLastIndex((x: { a: number }) => x.a === 4)(xs), none)
 *
 *
 * @since 3.0.0
 */
export const findLastIndex = (predicate) => (as) => {
    const len = as.length;
    for (let i = len - 1; i >= 0; i--) {
        if (predicate(as[i])) {
            return _.some(i);
        }
    }
    return _.none;
};
/**
 * Insert an element at the specified index, creating a new `ReadonlyArray`, or returning `None` if the index is out of bounds.
 *
 * @example
 * import { insertAt } from 'fp-ts/ReadonlyArray'
 * import { some } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(insertAt(2, 5)([1, 2, 3, 4]), some([1, 2, 5, 3, 4]))
 *
 * @since 3.0.0
 */
export const insertAt = (i, a) => (as) => {
    if (i < 0 || i > as.length) {
        return _.none;
    }
    if (isNonEmpty(as)) {
        const out = _.fromReadonlyNonEmptyArray(as);
        out.splice(i, 0, a);
        return _.some(out);
    }
    return _.some([a]);
};
/**
 * Change the element at the specified index, creating a new `ReadonlyArray`, or returning `None` if the index is out of bounds.
 *
 * @example
 * import { updateAt } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(updateAt(1, 1)([1, 2, 3]), some([1, 1, 3]))
 * assert.deepStrictEqual(updateAt(1, 1)([]), none)
 *
 * @since 3.0.0
 */
export const updateAt = (i, a) => modifyAt(i, () => a);
/**
 * Apply a function to the element at the specified index, creating a new `ReadonlyArray`, or returning `None` if the index is out
 * of bounds.
 *
 * @example
 * import { modifyAt } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * const double = (x: number): number => x * 2
 * assert.deepStrictEqual(modifyAt(1, double)([1, 2, 3]), some([1, 4, 3]))
 * assert.deepStrictEqual(modifyAt(1, double)([]), none)
 *
 * @since 3.0.0
 */
export const modifyAt = (i, f) => (as) => {
    if (isOutOfBound(i, as)) {
        return _.none;
    }
    const prev = as[i];
    const next = f(prev);
    if (next === prev) {
        return _.some(as);
    }
    const out = as.slice();
    out[i] = next;
    return _.some(out);
};
const unsafeDeleteAt = (i, as) => {
    const xs = as.slice();
    xs.splice(i, 1);
    return xs;
};
/**
 * Delete the element at the specified index, creating a new `ReadonlyArray`, or returning `None` if the index is out of bounds.
 *
 * @example
 * import { deleteAt } from 'fp-ts/ReadonlyArray'
 * import { some, none } from 'fp-ts/Option'
 *
 * assert.deepStrictEqual(deleteAt(0)([1, 2, 3]), some([2, 3]))
 * assert.deepStrictEqual(deleteAt(1)([]), none)
 *
 * @since 3.0.0
 */
export const deleteAt = (i) => (as) => isOutOfBound(i, as) ? _.none : _.some(unsafeDeleteAt(i, as));
/**
 * Reverse a `ReadonlyArray`, creating a new `ReadonlyArray`.
 *
 * @example
 * import { reverse } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(reverse([1, 2, 3]), [3, 2, 1])
 *
 * @category combinators
 * @since 3.0.0
 */
export const reverse = (as) => (as.length <= 1 ? as : as.slice().reverse());
/**
 * Extracts from a `ReadonlyArray` of `Either`s all the `Right` elements.
 *
 * @example
 * import { rights } from 'fp-ts/ReadonlyArray'
 * import { right, left } from 'fp-ts/Either'
 *
 * assert.deepStrictEqual(rights([right(1), left('foo'), right(2)]), [1, 2])
 *
 * @category combinators
 * @since 3.0.0
 */
export const rights = (as) => {
    const len = as.length;
    const out = [];
    for (let i = 0; i < len; i++) {
        const a = as[i];
        if (_.isRight(a)) {
            out.push(a.right);
        }
    }
    return out;
};
/**
 * Extracts from a `ReadonlyArray` of `Either` all the `Left` elements. All the `Left` elements are extracted in order
 *
 * @example
 * import { lefts } from 'fp-ts/ReadonlyArray'
 * import { left, right } from 'fp-ts/Either'
 *
 * assert.deepStrictEqual(lefts([right(1), left('foo'), right(2)]), ['foo'])
 *
 * @category combinators
 * @since 3.0.0
 */
export const lefts = (as) => {
    const out = [];
    const len = as.length;
    for (let i = 0; i < len; i++) {
        const a = as[i];
        if (_.isLeft(a)) {
            out.push(a.left);
        }
    }
    return out;
};
/**
 * Sort the elements of a `ReadonlyArray` in increasing order, creating a new `ReadonlyArray`.
 *
 * @example
 * import { sort } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 *
 * assert.deepStrictEqual(sort(N.Ord)([3, 2, 1]), [1, 2, 3])
 *
 * @category combinators
 * @since 3.0.0
 */
export const sort = (O) => (as) => as.length <= 1 ? as : as.slice().sort((first, second) => O.compare(second)(first));
/**
 * Apply a function to pairs of elements at the same index in two `ReadonlyArray`s, collecting the results in a new `ReadonlyArray`. If one
 * input `ReadonlyArray` is short, excess elements of the longer `ReadonlyArray` are discarded.
 *
 * @example
 * import { zipWith } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3], zipWith(['a', 'b', 'c', 'd'], (n, s) => s + n)), ['a1', 'b2', 'c3'])
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipWith = (fb, f) => (fa) => {
    const out = [];
    const len = Math.min(fa.length, fb.length);
    for (let i = 0; i < len; i++) {
        out[i] = f(fa[i], fb[i]);
    }
    return out;
};
/**
 * Takes two `ReadonlyArray`s and returns a `ReadonlyArray` of corresponding pairs. If one input `ReadonlyArray` is short, excess elements of the
 * longer `ReadonlyArray` are discarded.
 *
 * @example
 * import { zip } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3], zip(['a', 'b', 'c', 'd'])), [[1, 'a'], [2, 'b'], [3, 'c']])
 *
 * @category combinators
 * @since 3.0.0
 */
export const zip = (bs) => zipWith(bs, (a, b) => [a, b]);
/**
 * This function is the inverse of `zip`. Takes a `ReadonlyArray` of pairs and return two corresponding `ReadonlyArray`s.
 *
 * @example
 * import { unzip } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(unzip([[1, 'a'], [2, 'b'], [3, 'c']]), [[1, 2, 3], ['a', 'b', 'c']])
 *
 * @since 3.0.0
 */
export const unzip = (as) => {
    const fa = [];
    const fb = [];
    for (let i = 0; i < as.length; i++) {
        fa[i] = as[i][0];
        fb[i] = as[i][1];
    }
    return [fa, fb];
};
/**
 * Prepend an element to every member of a `ReadonlyArray`
 *
 * @example
 * import { prependAll } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3, 4], prependAll(9)), [9, 1, 9, 2, 9, 3, 9, 4])
 *
 * @category combinators
 * @since 3.0.0
 */
export const prependAll = (middle) => {
    const f = readonlyNonEmptyArray.prependAll(middle);
    return (as) => (isNonEmpty(as) ? f(as) : as);
};
/**
 * Places an element in between members of a `ReadonlyArray`
 *
 * @example
 * import { intersperse } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2, 3, 4], intersperse(9)), [1, 9, 2, 9, 3, 9, 4])
 *
 * @category combinators
 * @since 3.0.0
 */
export const intersperse = (middle) => {
    const f = readonlyNonEmptyArray.intersperse(middle);
    return (as) => (isNonEmpty(as) ? f(as) : as);
};
/**
 * Rotate a `ReadonlyArray` by `n` steps.
 *
 * @example
 * import { rotate } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(rotate(2)([1, 2, 3, 4, 5]), [4, 5, 1, 2, 3])
 * assert.deepStrictEqual(rotate(-2)([1, 2, 3, 4, 5]), [3, 4, 5, 1, 2])
 *
 * @category combinators
 * @since 3.0.0
 */
export const rotate = (n) => {
    const f = readonlyNonEmptyArray.rotate(n);
    return (as) => (isNonEmpty(as) ? f(as) : as);
};
/**
 * Tests whether a value is a member of a `ReadonlyArray`.
 *
 * @example
 * import { elem } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.strictEqual(pipe([1, 2, 3], elem(N.Eq)(2)), true)
 * assert.strictEqual(pipe([1, 2, 3], elem(N.Eq)(0)), false)
 *
 * @since 3.0.0
 */
export const elem = (E) => (a) => (as) => {
    const predicate = E.equals(a);
    let i = 0;
    const len = as.length;
    for (; i < len; i++) {
        if (predicate(as[i])) {
            return true;
        }
    }
    return false;
};
/**
 * Remove duplicates from a `ReadonlyArray`, keeping the first occurrence of an element.
 *
 * @example
 * import { uniq } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 *
 * assert.deepStrictEqual(uniq(N.Eq)([1, 2, 1]), [1, 2])
 *
 * @category combinators
 * @since 3.0.0
 */
export const uniq = (E) => {
    const f = readonlyNonEmptyArray.uniq(E);
    return (as) => (isNonEmpty(as) ? f(as) : as);
};
/**
 * Sort the elements of a `ReadonlyArray` in increasing order, where elements are compared using first `ords[0]`, then `ords[1]`,
 * etc...
 *
 * @example
 * import { sortBy } from 'fp-ts/ReadonlyArray'
 * import { contramap } from 'fp-ts/Ord'
 * import * as S from 'fp-ts/string'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * interface Person {
 *   name: string
 *   age: number
 * }
 * const byName = pipe(S.Ord, contramap((p: Person) => p.name))
 * const byAge = pipe(N.Ord, contramap((p: Person) => p.age))
 *
 * const sortByNameByAge = sortBy([byName, byAge])
 *
 * const persons = [{ name: 'a', age: 1 }, { name: 'b', age: 3 }, { name: 'c', age: 2 }, { name: 'b', age: 2 }]
 * assert.deepStrictEqual(sortByNameByAge(persons), [
 *   { name: 'a', age: 1 },
 *   { name: 'b', age: 2 },
 *   { name: 'b', age: 3 },
 *   { name: 'c', age: 2 }
 * ])
 *
 * @category combinators
 * @since 3.0.0
 */
export const sortBy = (ords) => {
    const f = readonlyNonEmptyArray.sortBy(ords);
    return (as) => (isNonEmpty(as) ? f(as) : as);
};
/**
 * A useful recursion pattern for processing a `ReadonlyArray` to produce a new `ReadonlyArray`, often used for "chopping" up the input
 * `ReadonlyArray`. Typically chop is called with some function that will consume an initial prefix of the `ReadonlyArray` and produce a
 * value and the rest of the `ReadonlyArray`.
 *
 * @example
 * import { Eq } from 'fp-ts/Eq'
 * import * as N from 'fp-ts/number'
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const group = <A>(E: Eq<A>): ((as: ReadonlyArray<A>) => ReadonlyArray<ReadonlyArray<A>>) => {
 *   return RA.chop(as => pipe(as, RA.spanLeft(E.equals(as[0]))))
 * }
 * assert.deepStrictEqual(group(N.Eq)([1, 1, 2, 3, 3, 4]), [[1, 1], [2], [3, 3], [4]])
 *
 * @category combinators
 * @since 3.0.0
 */
export const chop = (f) => {
    const g = readonlyNonEmptyArray.chop(f);
    return (as) => (isNonEmpty(as) ? g(as) : empty);
};
/**
 * Splits a `ReadonlyArray` into two pieces, the first piece has max `n` elements.
 *
 * @example
 * import { splitAt } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(splitAt(2)([1, 2, 3, 4, 5]), [[1, 2], [3, 4, 5]])
 *
 * @category combinators
 * @since 3.0.0
 */
export const splitAt = (n) => (as) => n >= 1 && isNonEmpty(as) ? readonlyNonEmptyArray.splitAt(n)(as) : isEmpty(as) ? [as, empty] : [empty, as];
/**
 * Splits a `ReadonlyArray` into length-`n` pieces. The last piece will be shorter if `n` does not evenly divide the length of
 * the `ReadonlyArray`. Note that `chunksOf(n)([])` is `[]`, not `[[]]`. This is intentional, and is consistent with a recursive
 * definition of `chunksOf`; it satisfies the property that
 *
 * ```ts
 * chunksOf(n)(xs).concat(chunksOf(n)(ys)) == chunksOf(n)(xs.concat(ys)))
 * ```
 *
 * whenever `n` evenly divides the length of `as`.
 *
 * @example
 * import { chunksOf } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(chunksOf(2)([1, 2, 3, 4, 5]), [[1, 2], [3, 4], [5]])
 *
 * @since 3.0.0
 */
export const chunksOf = (n) => {
    const f = readonlyNonEmptyArray.chunksOf(n);
    return (as) => (isNonEmpty(as) ? f(as) : empty);
};
/**
 * Creates a `ReadonlyArray` of unique values, in order, from all given `ReadonlyArray`s using a `Eq` for equality comparisons.
 *
 * @example
 * import { union } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2], union(N.Eq)([2, 3])), [1, 2, 3])
 *
 * @category combinators
 * @since 3.0.0
 */
export const union = (E) => {
    const unionE = readonlyNonEmptyArray.union(E);
    return (second) => (first) => isNonEmpty(first) && isNonEmpty(second) ? unionE(second)(first) : isNonEmpty(first) ? first : second;
};
/**
 * Creates a `ReadonlyArray` of unique values that are included in all given `ReadonlyArray`s using a `Eq` for equality
 * comparisons. The order and references of result values are determined by the first `ReadonlyArray`.
 *
 * @example
 * import { intersection } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2], intersection(N.Eq)([2, 3])), [2])
 *
 * @category combinators
 * @since 3.0.0
 */
export const intersection = (E) => {
    const elemE = elem(E);
    return (second) => (first) => first.filter((a) => elemE(a)(second));
};
/**
 * Creates a `ReadonlyArray` of values not included in the other given `ReadonlyArray` using a `Eq` for equality
 * comparisons. The order and references of result values are determined by the first `ReadonlyArray`.
 *
 * @example
 * import { difference } from 'fp-ts/ReadonlyArray'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe([1, 2], difference(N.Eq)([2, 3])), [1])
 *
 * @category combinators
 * @since 3.0.0
 */
export const difference = (E) => {
    const elemE = elem(E);
    return (second) => (first) => first.filter((a) => !elemE(a)(second));
};
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = readonlyNonEmptyArray.of;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
/**
 * @category MonoidK
 * @since 3.0.0
 */
export const emptyK = () => empty;
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * In case of `ReadonlyArray` concatenates the inputs into a single array.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     [1, 2, 3],
 *     RA.combineK(() => [4, 5])
 *   ),
 *   [1, 2, 3, 4, 5]
 * )
 *
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = (second) => (self) => self.concat(second());
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = (fa) => flatMap((f) => pipe(fa, map(f)));
/**
 * @since 3.0.0
 */
export const flatMapWithIndex = (f) => (as) => {
    if (isEmpty(as)) {
        return empty;
    }
    const out = [];
    for (let i = 0; i < as.length; i++) {
        out.push(...f(i, as[i]));
    }
    return out;
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     [1, 2, 3],
 *     RA.flatMap((n) => [`a${n}`, `b${n}`])
 *   ),
 *   ['a1', 'b1', 'a2', 'b2', 'a3', 'b3']
 * )
 * assert.deepStrictEqual(
 *   pipe(
 *     [1, 2, 3],
 *     RA.flatMap(() => [])
 *   ),
 *   []
 * )
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = (f) => flatMapWithIndex((_, a) => f(a));
/**
 * Removes one level of nesting
 *
 * Derivable from `Flattenable`.
 *
 * @example
 * import { flatten } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(flatten([[1], [2, 3], [4]]), [1, 2, 3, 4])
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => fa.map((a) => f(a)); // <= intended eta expansion
/**
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export const mapWithIndex = (f) => (fa) => fa.map((a, i) => f(i, a));
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export const filterMapWithIndex = (f) => (fa) => {
    const out = [];
    for (let i = 0; i < fa.length; i++) {
        const optionB = f(i, fa[i]);
        if (_.isSome(optionB)) {
            out.push(optionB.value);
        }
    }
    return out;
};
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = (f) => filterMapWithIndex((_, a) => f(a));
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = /*#__PURE__*/ filterMap(identity);
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = (fa) => {
    const left = [];
    const right = [];
    for (const e of fa) {
        if (_.isLeft(e)) {
            left.push(e.left);
        }
        else {
            right.push(e.right);
        }
    }
    return [left, right];
};
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = (f) => partitionMapWithIndex((_, a) => f(a));
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export const partitionMapWithIndex = (f) => (fa) => {
    const left = [];
    const right = [];
    for (let i = 0; i < fa.length; i++) {
        const e = f(i, fa[i]);
        if (_.isLeft(e)) {
            left.push(e.left);
        }
        else {
            right.push(e.right);
        }
    }
    return [left, right];
};
/**
 * @category Extendable
 * @since 3.0.0
 */
export const extend = (f) => (wa) => wa.map((_, i, as) => f(as.slice(i)));
/**
 * Derivable from `Extendable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const duplicate = /*#__PURE__*/ extend(identity);
/**
 * @category FoldableWithIndex
 * @since 3.0.0
 */
export const foldMapWithIndex = (M) => (f) => (fa) => fa.reduce((b, a, i) => M.combine(f(i, a))(b), M.empty);
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduce = (b, f) => reduceWithIndex(b, (_, b, a) => f(b, a));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const foldMap = (M) => {
    const foldMapWithIndexM = foldMapWithIndex(M);
    return (f) => foldMapWithIndexM((_, a) => f(a));
};
/**
 * @category FoldableWithIndex
 * @since 3.0.0
 */
export const reduceWithIndex = (b, f) => (fa) => {
    const len = fa.length;
    let r = b;
    for (let i = 0; i < len; i++) {
        r = f(i, r, fa[i]);
    }
    return r;
};
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (b, f) => reduceRightWithIndex(b, (_, a, b) => f(a, b));
/**
 * @category FoldableWithIndex
 * @since 3.0.0
 */
export const reduceRightWithIndex = (b, f) => (fa) => fa.reduceRight((b, a, i) => f(i, a, b), b);
/**
 * @category Traversable
 * @since 3.0.0
 */
export const traverse = (F) => {
    const traverseWithIndexF = traverseWithIndex(F);
    return (f) => traverseWithIndexF((_, a) => f(a));
};
/**
 * @category TraversableWithIndex
 * @since 3.0.0
 */
export const traverseWithIndex = (F) => (f) => {
    return reduceWithIndex(F.of(emptyK()), (i, fbs, a) => pipe(fbs, F.map((bs) => (b) => append(b)(bs)), F.ap(f(i, a))));
};
/**
 * @category Unfoldable
 * @since 3.0.0
 */
export const unfold = (b, f) => {
    const out = [];
    let next = b;
    let o;
    while (_.isSome((o = f(next)))) {
        const [a, b] = o.value;
        out.push(a);
        next = b;
    }
    return out;
};
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => ({
    show: (as) => `[${as.map(S.show).join(', ')}]`
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionSemigroup = (E) => ({
    combine: union(E)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionMonoid = (E) => ({
    combine: getUnionSemigroup(E).combine,
    empty
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getIntersectionSemigroup = (E) => ({
    combine: intersection(E)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getDifferenceMagma = (E) => ({
    combine: difference(E)
});
/**
 * Returns a `Semigroup` for `ReadonlyArray<A>`.
 *
 * @example
 * import { getSemigroup } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const S = getSemigroup<number>()
 * assert.deepStrictEqual(pipe([1, 2], S.combine([3, 4])), [1, 2, 3, 4])
 *
 * @category instances
 * @since 3.0.0
 */
export const getSemigroup = () => ({
    combine: concat
});
/**
 * Returns a `Monoid` for `ReadonlyArray<A>`.
 *
 * @category instances
 * @since 3.0.0
 */
export const getMonoid = () => ({
    combine: getSemigroup().combine,
    empty
});
/**
 * Derives an `Eq` over the `ReadonlyArray` of a given element type from the `Eq` of that type. The derived `Eq` defines two
 * `ReadonlyArray`s as equal if all elements of both `ReadonlyArray`s are compared equal pairwise with the given `E`. In case of `ReadonlyArray`s of
 * different lengths, the result is non equality.
 *
 * @example
 * import * as S from 'fp-ts/string'
 * import { getEq } from 'fp-ts/ReadonlyArray'
 *
 * const E = getEq(S.Eq)
 * assert.strictEqual(E.equals(['a', 'b'])(['a', 'b']), true)
 * assert.strictEqual(E.equals(['a'])([]), false)
 *
 * @category instances
 * @since 3.0.0
 */
export const getEq = (E) => eq.fromEquals((second) => (first) => first.length === second.length && first.every((x, i) => E.equals(second[i])(x)));
/**
 * Derives an `Ord` over the `ReadonlyArray` of a given element type from the `Ord` of that type. The ordering between two such
 * `ReadonlyArray`s is equal to: the first non equal comparison of each `ReadonlyArray`s elements taken pairwise in increasing order, in
 * case of equality over all the pairwise elements; the longest `ReadonlyArray` is considered the greatest, if both `ReadonlyArray`s have
 * the same length, the result is equality.
 *
 * @example
 * import { getOrd } from 'fp-ts/ReadonlyArray'
 * import * as S from 'fp-ts/string'
 * import { pipe } from 'fp-ts/function'
 *
 * const O = getOrd(S.Ord)
 * assert.strictEqual(pipe(['b'], O.compare(['a'])), 1)
 * assert.strictEqual(pipe(['a'], O.compare(['a'])), 0)
 * assert.strictEqual(pipe(['a'], O.compare(['b'])), -1)
 *
 *
 * @category instances
 * @since 3.0.0
 */
export const getOrd = (O) => ord.fromCompare((second) => (first) => {
    const aLen = first.length;
    const bLen = second.length;
    const len = Math.min(aLen, bLen);
    for (let i = 0; i < len; i++) {
        const o = O.compare(second[i])(first[i]);
        if (o !== 0) {
            return o;
        }
    }
    return number.Ord.compare(bLen)(aLen);
});
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FunctorWithIndex = {
    mapWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = 
/*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = 
/*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @example
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     [1, 2, 3],
 *     RA.tap(() => ['a', 'b'])
 *   ),
 *   [1, 1, 2, 2, 3, 3]
 * )
 * assert.deepStrictEqual(
 *   pipe(
 *     [1, 2, 3],
 *     RA.tap(() => [])
 *   ),
 *   []
 * )
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = 
/*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const Unfoldable = {
    unfold
};
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
/**
 * @category instances
 * @since 3.0.0
 */
export const MonoidK = {
    combineK,
    emptyK
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const guard = /*#__PURE__*/ monoidK.guard(MonoidK, Pointed);
/**
 * @category instances
 * @since 3.0.0
 */
export const Extendable = {
    map,
    extend
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Compactable = {
    compact,
    separate
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Filterable = {
    filterMap,
    partitionMap
};
// TODO: perf
/**
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ filterable.filter(Filterable);
/**
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ filterable.partition(Filterable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FilterableWithIndex = {
    partitionMapWithIndex,
    filterMapWithIndex
};
/**
 * @since 3.0.0
 */
export const filterWithIndex = /*#__PURE__*/ filterableWithIndex.filterWithIndex(FilterableWithIndex);
/**
 * @since 3.0.0
 */
export const partitionWithIndex = /*#__PURE__*/ filterableWithIndex.partitionWithIndex(FilterableWithIndex);
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FoldableWithIndex = {
    reduceWithIndex,
    foldMapWithIndex,
    reduceRightWithIndex
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    traverse
};
/**
 * @since 3.0.0
 */
export const sequence = 
/*#__PURE__*/ traversable.getDefaultSequence(traverse);
/**
 * @category instances
 * @since 3.0.0
 */
export const TraversableWithIndex = {
    traverseWithIndex
};
/**
 * @category FilterableWithEffect
 * @since 3.0.0
 */
export const filterMapWithEffect = 
/*#__PURE__*/ filterableWithEffect.getDefaultFilterMapWithEffect(Traversable, Compactable);
/**
 * @category FilterableWithEffect
 * @since 3.0.0
 */
export const partitionMapWithEffect = 
/*#__PURE__*/ filterableWithEffect.getDefaultPartitionMapWithEffect(Traversable, Compactable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FilterableWithEffect = {
    filterMapWithEffect,
    partitionMapWithEffect
};
/**
 * Filter values inside a context.
 *
 * @example
 * import { pipe } from 'fp-ts/function'
 * import * as RA from 'fp-ts/ReadonlyArray'
 * import * as T from 'fp-ts/Task'
 *
 * const filterWithEffect = RA.filterWithEffect(T.ApplicativePar)
 * async function test() {
 *   assert.deepStrictEqual(
 *     await pipe(
 *       [-1, 2, 3],
 *       filterWithEffect((n) => T.of(n > 0))
 *     )(),
 *     [2, 3]
 *   )
 * }
 * test()
 *
 * @category combinators
 * @since 3.0.0
 */
export const filterWithEffect = 
/*#__PURE__*/ filterableWithEffect.filterWithEffect(FilterableWithEffect);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partitionWithEffect = 
/*#__PURE__*/ filterableWithEffect.partitionWithEffect(FilterableWithEffect);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromOption = {
    fromOption
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = 
/*#__PURE__*/ fromOption_.fromPredicate(FromOption);
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromRefinement = /*#__PURE__*/ fromOption_.fromRefinement(FromOption);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromOption_.fromOptionK(FromOption);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/ fromOption_.fromNullable(FromOption);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromOption_.fromNullableK(FromOption);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = /*#__PURE__*/ fromOption_.flatMapNullableK(FromOption, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category FlattenableRec
 * @since 3.0.0
 */
export const flatMapRecDepthFirst = (f) => (a) => {
    const abs = [...f(a)];
    const out = [];
    while (abs.length > 0) {
        const e = abs.shift();
        if (_.isLeft(e)) {
            abs.unshift(...f(e.left));
        }
        else {
            out.push(e.right);
        }
    }
    return out;
};
/**
 * @category FlattenableRec
 * @since 3.0.0
 */
export const flatMapRecBreadthFirst = (f) => (a) => {
    const initial = f(a);
    const abs = [];
    const out = [];
    function go(e) {
        if (_.isLeft(e)) {
            f(e.left).forEach((v) => abs.push(v));
        }
        else {
            out.push(e.right);
        }
    }
    for (const e of initial) {
        go(e);
    }
    while (abs.length > 0) {
        go(abs.shift());
    }
    return out;
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FlattenableRecDepthFirst = {
    flatMapRec: flatMapRecDepthFirst
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FlattenableRecBreadthFirst = {
    flatMapRec: flatMapRecBreadthFirst
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * An empty `ReadonlyArray`.
 *
 * @since 3.0.0
 */
export const empty = _.emptyReadonlyArray;
export function every(predicate) {
    return (as) => as.every(predicate);
}
/**
 * Check if a predicate holds true for any `ReadonlyArray` member.
 *
 * @example
 * import { some } from 'fp-ts/ReadonlyArray'
 * import { pipe } from 'fp-ts/function'
 *
 * const isPositive = (n: number): boolean => n > 0
 *
 * assert.deepStrictEqual(pipe([-1, -2, 3], some(isPositive)), true)
 * assert.deepStrictEqual(pipe([-1, -2, -3], some(isPositive)), false)
 *
 * @since 3.0.0
 */
export const some = (predicate) => (as) => as.some(predicate);
/**
 * Alias of [`some`](#some)
 *
 * @since 3.0.0
 */
export const exists = some;
/**
 * Places an element in between members of a `ReadonlyArray`, then folds the results using the provided `Monoid`.
 *
 * @example
 * import * as S from 'fp-ts/string'
 * import { intercalate } from 'fp-ts/ReadonlyArray'
 *
 * assert.deepStrictEqual(intercalate(S.Monoid)('-')(['a', 'b', 'c']), 'a-b-c')
 *
 * @since 3.0.0
 */
export const intercalate = (M) => {
    const intercalateM = readonlyNonEmptyArray.intercalate(M);
    return (middle) => match(() => M.empty, intercalateM(middle));
};
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = /*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/ apply.apT(Apply);
