import * as eq from './Eq';
import { identity } from './function';
import * as _ from './internal';
import * as predicate from './Predicate';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Create a `ReadonlySet` from one element.
 *
 * @category constructors
 * @since 3.0.0
 */
export const singleton = (a) => new Set([a]);
/**
 * Create a `ReadonlySet` from a `ReadonlyArray`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromReadonlyArray = (E) => (as) => {
    const len = as.length;
    const r = new Set();
    const has = elem(E);
    for (let i = 0; i < len; i++) {
        const a = as[i];
        if (!has(a)(r)) {
            r.add(a);
        }
    }
    return r;
};
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Checks an element is a member of a set;
 * If yes, removes the value from the set
 * If no, inserts the value to the set
 *
 * @category combinators
 * @since 3.0.0
 */
export const toggle = (E) => {
    const elemE = elem(E);
    const removeE = remove(E);
    const insertE = insert(E);
    return (a) => {
        const elemEa = elemE(a);
        return (set) => (elemEa(set) ? removeE : insertE)(a)(set);
    };
};
/**
 * Return the union of two `ReadonlySet`s.
 *
 * @category combinators
 * @since 3.0.0
 */
export const union = (E) => {
    const elemE = elem(E);
    return (second) => (first) => {
        if (isEmpty(first)) {
            return second;
        }
        if (isEmpty(second)) {
            return first;
        }
        const r = new Set(first);
        second.forEach((e) => {
            if (!elemE(e)(r)) {
                r.add(e);
            }
        });
        return r;
    };
};
/**
 * The `ReadonlySet` of elements which are in both the first and second `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const intersection = (E) => {
    const elemE = elem(E);
    return (second) => (first) => {
        if (isEmpty(first) || isEmpty(second)) {
            return empty;
        }
        const r = new Set();
        first.forEach((e) => {
            if (elemE(e)(second)) {
                r.add(e);
            }
        });
        return r;
    };
};
/**
 * Return the set difference (`x` - `y`).
 *
 * @example
 * import { difference } from 'fp-ts/ReadonlySet'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(new Set([1, 2]), difference(N.Eq)(new Set([1, 3]))), new Set([2]))
 *
 * @category combinators
 * @since 3.0.0
 */
export const difference = (E) => {
    const elemE = elem(E);
    return (second) => filter((a) => !elemE(a)(second));
};
/**
 * Insert a value into a `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const insert = (E) => {
    const elemE = elem(E);
    return (a) => (set) => {
        if (!elemE(a)(set)) {
            const r = new Set(set);
            r.add(a);
            return r;
        }
        else {
            return set;
        }
    };
};
/**
 * Delete a value from a `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const remove = (E) => (a) => filter(predicate.not(E.equals(a)));
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
export const map = (E) => {
    const elemE = elem(E);
    return (f) => (set) => {
        const r = new Set();
        set.forEach((e) => {
            const v = f(e);
            if (!elemE(v)(r)) {
                r.add(v);
            }
        });
        return r;
    };
};
/**
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = (E) => {
    const elemE = elem(E);
    return (f) => (set) => {
        const r = new Set();
        set.forEach((e) => {
            f(e).forEach((e) => {
                if (!elemE(e)(r)) {
                    r.add(e);
                }
            });
        });
        return r;
    };
};
/**
 * @category Compactable
 * @since 3.0.0
 */
export const compact = (E) => filterMap(E)(identity);
/**
 * @category Compactable
 * @since 3.0.0
 */
export const separate = (EE, EA) => (fa) => {
    const elemEE = elem(EE);
    const elemEA = elem(EA);
    const left = new Set();
    const right = new Set();
    fa.forEach((e) => {
        switch (e._tag) {
            case 'Left':
                if (!elemEE(e.left)(left)) {
                    left.add(e.left);
                }
                break;
            case 'Right':
                if (!elemEA(e.right)(right)) {
                    right.add(e.right);
                }
                break;
        }
    });
    return [left, right];
};
export function filter(predicate) {
    return (s) => {
        const values = s.values();
        let e;
        const r = new Set();
        while (!(e = values.next()).done) {
            const a = e.value;
            if (predicate(a)) {
                r.add(a);
            }
        }
        return r;
    };
}
/**
 * @category Filterable
 * @since 3.0.0
 */
export const filterMap = (E) => {
    const elemE = elem(E);
    return (f) => (fa) => {
        const r = new Set();
        fa.forEach((a) => {
            const ob = f(a);
            if (_.isSome(ob) && !elemE(ob.value)(r)) {
                r.add(ob.value);
            }
        });
        return r;
    };
};
export function partition(predicate) {
    return (s) => {
        const values = s.values();
        let e;
        const right = new Set();
        const left = new Set();
        while (!(e = values.next()).done) {
            const a = e.value;
            if (predicate(a)) {
                right.add(a);
            }
            else {
                left.add(a);
            }
        }
        return [left, right];
    };
}
/**
 * @category Filterable
 * @since 3.0.0
 */
export const partitionMap = (EB, EC) => (f) => (s) => {
    const values = s.values();
    let e;
    const left = new Set();
    const right = new Set();
    const hasB = elem(EB);
    const hasC = elem(EC);
    while (!(e = values.next()).done) {
        const v = f(e.value);
        switch (v._tag) {
            case 'Left':
                if (!hasB(v.left)(left)) {
                    left.add(v.left);
                }
                break;
            case 'Right':
                if (!hasC(v.right)(right)) {
                    right.add(v.right);
                }
                break;
        }
    }
    return [left, right];
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => ({
    show: (s) => {
        const entries = [];
        s.forEach((a) => {
            entries.push(S.show(a));
        });
        return `new Set([${entries.sort().join(', ')}])`;
    }
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = (E) => {
    const subsetE = isSubset(E);
    return eq.fromEquals((second) => (first) => subsetE(first)(second) && subsetE(second)(first));
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionSemigroup = (E) => ({
    combine: union(E)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getUnionMonoid = (E) => ({
    combine: getUnionSemigroup(E).combine,
    empty
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getIntersectionSemigroup = (E) => ({
    combine: intersection(E)
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getDifferenceMagma = (E) => ({
    combine: difference(E)
});
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * An empty `ReadonlySet`.
 *
 * @since 3.0.0
 */
export const empty = new Set();
/**
 * Test whether a `ReadonlySet` is empty.
 *
 * @since 3.0.0
 */
export const isEmpty = (set) => set.size === 0;
/**
 * Calculate the number of elements in a `ReadonlySet`.
 *
 * @since 3.0.0
 */
export const size = (set) => set.size;
/**
 * @since 3.0.0
 */
export const some = (predicate) => (s) => {
    const values = s.values();
    let e;
    let found = false;
    while (!found && !(e = values.next()).done) {
        found = predicate(e.value);
    }
    return found;
};
export function every(p) {
    return predicate.not(some(predicate.not(p)));
}
/**
 * Return `true` if and only if every element in the first `ReadonlySet` is an element of the second `ReadonlySet`.
 *
 * @since 3.0.0
 */
export const isSubset = (E) => {
    const elemE = elem(E);
    return (second) => every((a) => elemE(a)(second));
};
/**
 * Tests whether a value is a member of a `ReadonlySet`.
 *
 * @since 3.0.0
 */
export const elem = (E) => (a) => {
    const predicate = E.equals(a);
    return (set) => {
        const values = set.values();
        let e;
        let found = false;
        while (!found && !(e = values.next()).done) {
            found = predicate(e.value);
        }
        return found;
    };
};
/**
 * @since 3.0.0
 */
export const reduce = (O) => {
    const toReadonlyArrayO = toReadonlyArray(O);
    return (b, f) => (fa) => toReadonlyArrayO(fa).reduce(f, b);
};
/**
 * @since 3.0.0
 */
export const foldMap = (O) => (M) => {
    const toReadonlyArrayO = toReadonlyArray(O);
    return (f) => (fa) => toReadonlyArrayO(fa).reduce((b, a) => M.combine(f(a))(b), M.empty);
};
/**
 * @since 3.0.0
 */
export const reduceRight = (O) => {
    const toReadonlyArrayO = toReadonlyArray(O);
    return (b, f) => (fa) => toReadonlyArrayO(fa).reduceRight((b, a) => f(a, b), b);
};
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlySet`.
 *
 * @since 3.0.0
 */
export const toReadonlyArray = (O) => (s) => {
    const out = [];
    s.forEach((e) => out.push(e));
    return out.sort((first, second) => O.compare(second)(first));
};
