import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as compactable from './Compactable';
import * as either from './Either';
import * as eitherT from './EitherT';
import * as fromEither_ from './FromEither';
import * as fromIO_ from './FromIO';
import * as fromTask_ from './FromTask';
import { flow, identity, pipe, SK } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as task from './Task';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const left = /*#__PURE__*/ eitherT.left(task.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const right = /*#__PURE__*/ eitherT.right(task.Pointed);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightTask = /*#__PURE__*/ eitherT.rightF(task.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftTask = /*#__PURE__*/ eitherT.leftF(task.Functor);
/**
 * @category constructors
 * @since 3.0.0
 */
export const rightIO = /*#__PURE__*/ flow(task.fromIO, rightTask);
/**
 * @category constructors
 * @since 3.0.0
 */
export const leftIO = /*#__PURE__*/ flow(task.fromIO, leftTask);
// -------------------------------------------------------------------------------------
// natural transformations
// -------------------------------------------------------------------------------------
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIO = rightIO;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTask = rightTask;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromEither = task.of;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromIOEither = task.fromIO;
/**
 * @category natural transformations
 * @since 3.0.0
 */
export const fromTaskOption = (onNone) => task.map(either.fromOption(onNone));
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * @category destructors
 * @since 3.0.0
 */
export const match = /*#__PURE__*/ eitherT.match(task.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const matchWithEffect = /*#__PURE__*/ eitherT.matchWithEffect(task.Monad);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElse = 
/*#__PURE__*/ eitherT.getOrElse(task.Functor);
/**
 * @category destructors
 * @since 3.0.0
 */
export const getOrElseE = 
/*#__PURE__*/ eitherT.getOrElseE(task.Monad);
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * Transforms a `Promise` that may reject to a `Promise` that never rejects and returns an `Either` instead.
 *
 * See also [`tryCatchK`](#trycatchk).
 *
 * @example
 * import * as E from 'fp-ts/Either'
 * import * as TE from 'fp-ts/TaskEither'
 * import { identity } from 'fp-ts/function'
 *
 * async function test() {
 *   assert.deepStrictEqual(await TE.tryCatch(() => Promise.resolve(1), identity)(), E.right(1))
 *   assert.deepStrictEqual(await TE.tryCatch(() => Promise.reject('error'), identity)(), E.left('error'))
 * }
 *
 * test()
 *
 * @category interop
 * @since 3.0.0
 */
export const tryCatch = (f, onRejected) => async () => {
    try {
        return await f().then(_.right);
    }
    catch (reason) {
        return _.left(onRejected(reason));
    }
};
/**
 * Converts a function returning a `Promise` that may reject to one returning a `TaskEither`.
 *
 * @category interop
 * @since 3.0.0
 */
export const tryCatchK = (f, onRejected) => (...a) => tryCatch(() => f(...a), onRejected);
/**
 * @category interop
 * @since 3.0.0
 */
export const toUnion = /*#__PURE__*/ eitherT.toUnion(task.Functor);
/**
 * Returns `ma` if is a `Right` or the value returned by `onError ` otherwise.
 *
 * See also [alt](#alt).
 *
 * @example
 * import * as E from 'fp-ts/Either'
 * import { pipe } from 'fp-ts/function'
 * import * as TE from 'fp-ts/TaskEither'
 *
 * async function test() {
 *   const errorHandler = TE.orElse((error: string) => TE.right(`recovering from ${error}...`))
 *   assert.deepStrictEqual(await pipe(TE.right('ok'), errorHandler)(), E.right('ok'))
 *   assert.deepStrictEqual(await pipe(TE.left('ko'), errorHandler)(), E.right('recovering from ko...'))
 * }
 *
 * test()
 *
 * @category combinators
 * @since 3.0.0
 */
export const orElse = /*#__PURE__*/ eitherT.orElse(task.Monad);
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = /*#__PURE__*/ eitherT.swap(task.Functor);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskOptionK = (onNone) => {
    const from = fromTaskOption(onNone);
    return (f) => flow(f, from);
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskOptionK = (onNone) => (f) => (ma) => pipe(ma, flatMap(fromTaskOptionK(onNone)(f)));
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOEitherK = (f) => flow(f, fromIOEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOEitherK = (f) => flatMap(fromIOEitherK(f));
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = /*#__PURE__*/ eitherT.map(task.Functor);
/**
 * Returns an effect whose failure and success channels have been mapped by
 * the specified pair of functions, `f` and `g`.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = 
/*#__PURE__*/ eitherT.mapBoth(task.Functor);
/**
 * Returns an effect with its error channel mapped using the specified
 * function. This can be used to lift a "smaller" error into a "larger" error.
 *
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapError = 
/*#__PURE__*/ eitherT.mapLeft(task.Functor);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = 
/*#__PURE__*/ eitherT.ap(task.ApplyPar);
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = /*#__PURE__*/ eitherT.flatMap(task.Monad);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = 
/*#__PURE__*/ flatMap(identity);
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * In case of `TaskEither` returns `first` if it is a `Right` or the value returned by `second` otherwise.
 *
 * See also [orElse](#orElse).
 *
 * @example
 * import * as E from 'fp-ts/Either'
 * import { pipe } from 'fp-ts/function'
 * import * as TE from 'fp-ts/TaskEither'
 *
 * async function test() {
 *   assert.deepStrictEqual(
 *     await pipe(
 *       TE.right(1),
 *       TE.combineK(() => TE.right(2))
 *     )(),
 *     E.right(1)
 *   )
 *   assert.deepStrictEqual(
 *     await pipe(
 *       TE.left('a'),
 *       TE.combineK(() => TE.right(2))
 *     )(),
 *     E.right(2)
 *   )
 *   assert.deepStrictEqual(
 *     await pipe(
 *       TE.left('a'),
 *       TE.combineK(() => TE.left('b'))
 *     )(),
 *     E.left('b')
 *   )
 * }
 *
 * test()
 *
 * @category SemigroupK
 * @since 3.0.0
 */
export const combineK = /*#__PURE__*/ eitherT.combineK(task.Monad);
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = right;
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * The default [`ApplicativePar`](#applicativepar) instance returns the first error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * See [`getValidatedApplicative`](./Either.ts.html#getvalidatedapplicative).
 *
 * @category instances
 * @since 3.0.0
 */
export const getValidatedApplicative = (A, S) => ({
    map,
    ap: apply.getApComposition(A, either.getValidatedApplicative(S)),
    of
});
/**
 * The default [`SemigroupK`](#semigroupk) instance returns the last error, if you want to
 * get all errors you need to provide an way to combine them via a `Semigroup`.
 *
 * @category instances
 * @since 3.0.0
 */
export const getValidatedSemigroupK = (S) => {
    return {
        combineK: eitherT.getValidatedCombineK(task.Monad, S)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getCompactable = (M) => {
    const C = either.getCompactable(M);
    const F = { map: either.map };
    return {
        compact: compactable.getCompactComposition(task.Functor, C),
        separate: compactable.getSeparateComposition(task.Functor, C, F)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFilterable = (M) => {
    return {
        partitionMap: (f) => partitionMap(f, () => M.empty),
        filterMap: (f) => filterMap(f, () => M.empty)
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplyPar = {
    map,
    ap
};
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = /*#__PURE__*/ apply.zipLeftPar(ApplyPar);
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = /*#__PURE__*/ apply.zipRightPar(ApplyPar);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativePar = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
const apSeq = /*#__PURE__*/ flattenable.ap(Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplySeq = {
    map,
    ap: apSeq
};
/**
 * @category instances
 * @since 3.0.0
 */
export const ApplicativeSeq = {
    map,
    ap: apSeq,
    of
};
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export const tap = /*#__PURE__*/ flattenable.tap(Flattenable);
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export const tapError = /*#__PURE__*/ eitherT.tapLeft(task.Monad);
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const SemigroupK = {
    combineK
};
/**
 * @category instances
 * @since 3.0.0
 */
export const FromIO = {
    fromIO
};
// -------------------------------------------------------------------------------------
// logging
// -------------------------------------------------------------------------------------
/**
 * @category logging
 * @since 3.0.0
 */
export const log = /*#__PURE__*/ fromIO_.log(FromIO);
/**
 * @category logging
 * @since 3.0.0
 */
export const logError = 
/*#__PURE__*/ fromIO_.logError(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromIOK = /*#__PURE__*/ fromIO_.fromIOK(FromIO);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapIOK = 
/*#__PURE__*/ fromIO_.flatMapIOK(FromIO, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromTask = {
    fromIO,
    fromTask
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromTaskK = /*#__PURE__*/ fromTask_.fromTaskK(FromTask);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapTaskK = 
/*#__PURE__*/ fromTask_.flatMapTaskK(FromTask, Flattenable);
/**
 * @category instances
 * @since 3.0.0
 */
export const FromEither = {
    fromEither
};
/**
 * Derivable from `FromEither`.
 *
 * @category natural transformations
 * @since 3.0.0
 */
export const fromOption = 
/*#__PURE__*/ fromEither_.fromOption(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionK = /*#__PURE__*/ fromEither_.fromOptionK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapOptionK = /*#__PURE__*/ fromEither_.flatMapOptionK(FromEither, Flattenable);
/**
 * Derivable from `FromEither`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicate = /*#__PURE__*/ fromEither_.fromPredicate(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filter = /*#__PURE__*/ fromEither_.filter(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterMap = /*#__PURE__*/ fromEither_.filterMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partition = /*#__PURE__*/ fromEither_.partition(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const partitionMap = /*#__PURE__*/ fromEither_.partitionMap(FromEither, Flattenable);
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = /*#__PURE__*/ fromEither_.fromEitherK(FromEither);
/**
 * @category combinators
 * @since 3.0.0
 */
export const flatMapEitherK = /*#__PURE__*/ fromEither_.flatMapEitherK(FromEither, Flattenable);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullable = 
/*#__PURE__*/ fromEither_.fromNullable(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableK = /*#__PURE__*/ fromEither_.fromNullableK(FromEither);
/**
 * @category interop
 * @since 3.0.0
 */
export const flatMapNullableK = 
/*#__PURE__*/ fromEither_.flatMapNullableK(FromEither, Flattenable);
export function taskify(f) {
    return function () {
        const args = Array.prototype.slice.call(arguments);
        return () => new Promise((resolve) => {
            const cbResolver = (e, r) => (e != null ? resolve(_.left(e)) : resolve(_.right(r)));
            f.apply(null, args.concat(cbResolver));
        });
    };
}
/**
 * Make sure that a resource is cleaned up in the event of an exception (\*). The release action is called regardless of
 * whether the body action throws (\*) or returns.
 *
 * (\*) i.e. returns a `Left`
 *
 * @since 3.0.0
 */
export const bracket = /*#__PURE__*/ eitherT.bracket(task.Monad);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = /*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(ApplyPar);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/ apply.apT(ApplyPar);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
// --- Par ---
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(ApplyPar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => flow(task.traverseReadonlyNonEmptyArrayWithIndex(f), task.map(either.traverseReadonlyNonEmptyArrayWithIndex(SK)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplyPar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (f) => {
    return traverseReadonlyArrayWithIndex((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativePar)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = 
/*#__PURE__*/ traverseReadonlyArray(identity);
// --- Seq ---
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplySeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndexSeq = (f) => (as) => () => _.tail(as).reduce((acc, a, i) => acc.then((ebs) => _.isLeft(ebs)
    ? acc
    : f(i + 1, a)().then((eb) => {
        if (_.isLeft(eb)) {
            return eb;
        }
        ebs.right.push(eb.right);
        return ebs;
    })), f(0, _.head(as))().then(either.map(_.singleton)));
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndexSeq = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndexSeq(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(ApplySeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArraySeq = (f) => {
    return traverseReadonlyNonEmptyArrayWithIndexSeq((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#traverse(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArraySeq = (f) => {
    return traverseReadonlyArrayWithIndexSeq((_, a) => f(a));
};
/**
 * Equivalent to `ReadonlyArray#sequence(ApplicativeSeq)`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArraySeq = 
/*#__PURE__*/ traverseReadonlyArraySeq(identity);
