import * as apply from './Apply';
import * as flattenable from './Flattenable';
import * as eq from './Eq';
import { flow, identity, pipe } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as readonlyArray from './ReadonlyArray';
import * as readonlyNonEmptyArray from './ReadonlyNonEmptyArray';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const make = (value, forest = readonlyArray.empty) => ({
    value,
    forest
});
/**
 * Build a (possibly infinite) tree from a seed value in breadth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export const unfoldTree = (f) => (b) => {
    const [a, bs] = f(b);
    return {
        value: a,
        forest: pipe(bs, unfoldForest(f))
    };
};
/**
 * Build a (possibly infinite) forest from a list of seed values in breadth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export const unfoldForest = (f) => (bs) => bs.map(unfoldTree(f));
/**
 * Monadic tree builder, in depth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export const unfoldTreeWithEffect = (M, A) => {
    const unfoldForestWithEffectMA = unfoldForestWithEffect(M, A);
    return (f) => flow(f, M.flatMap(([value, bs]) => pipe(bs, unfoldForestWithEffectMA(f), M.map((forest) => ({ value, forest })))));
};
/**
 * Monadic forest builder, in depth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export const unfoldForestWithEffect = (M, A) => {
    const traverseA = readonlyArray.traverse(A);
    return (f) => traverseA(unfoldTreeWithEffect(M, A)(f));
};
// -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------
/**
 * Fold a tree into a "summary" value in depth-first order.
 *
 * For each node in the tree, apply `f` to the `value` and the result of applying `f` to each `forest`.
 *
 * This is also known as the catamorphism on trees.
 *
 * @example
 * import { fold, make } from 'fp-ts/Tree'
 * import * as N from 'fp-ts/number'
 * import { combineAll } from 'fp-ts/Monoid'
 * import { pipe } from 'fp-ts/function'
 * import { isEmpty } from 'fp-ts/ReadonlyArray'
 *
 * const tree = make(1, [make(2), make(3)])
 *
 * const sum = combineAll(N.MonoidSum)
 *
 * assert.deepStrictEqual(pipe(tree, fold((a, bs) => a + sum(bs))), 6)
 * assert.deepStrictEqual(pipe(tree, fold((a, bs) => bs.reduce((b, acc) => Math.max(b, acc), a))), 3)
 * assert.deepStrictEqual(pipe(tree, fold((_, bs) => (isEmpty(bs) ? 1 : sum(bs)))), 2)
 *
 * @category destructors
 * @since 3.0.0
 */
export const fold = (f) => {
    const go = (tree) => f(tree.value, tree.forest.map(go));
    return go;
};
// -------------------------------------------------------------------------------------
// type class members
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => ({
    value: f(fa.value),
    forest: fa.forest.map(map(f))
});
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = (fa) => flatMap((f) => pipe(fa, map(f)));
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export const flatMap = (f) => (ma) => {
    const { value, forest } = f(ma.value);
    const combine = readonlyArray.getMonoid().combine;
    return {
        value,
        forest: combine(ma.forest.map(flatMap(f)))(forest)
    };
};
/**
 * @category Extendable
 * @since 3.0.0
 */
export const extend = (f) => (wa) => ({
    value: f(wa),
    forest: wa.forest.map(extend(f))
});
/**
 * Derivable from `Extendable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const duplicate = /*#__PURE__*/ extend(identity);
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ flatMap(identity);
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduce = (b, f) => (fa) => {
    let r = f(b, fa.value);
    const len = fa.forest.length;
    for (let i = 0; i < len; i++) {
        r = pipe(fa.forest[i], reduce(r, f));
    }
    return r;
};
/**
 * @category Foldable
 * @since 3.0.0
 */
export const foldMap = (M) => (f) => reduce(M.empty, (acc, a) => M.combine(f(a))(acc));
/**
 * @category Foldable
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (fa) => {
    let r = b;
    const len = fa.forest.length;
    for (let i = len - 1; i >= 0; i--) {
        r = pipe(fa.forest[i], reduceRight(r, f));
    }
    return f(fa.value, r);
};
/**
 * @category Extract
 * @since 3.0.0
 */
export const extract = (wa) => wa.value;
/**
 * @category Traversable
 * @since 3.0.0
 */
export const traverse = (F) => {
    const traverseF = readonlyNonEmptyArray.traverse(F);
    const out = (f) => (ta) => {
        const fb = f(ta.value);
        if (readonlyNonEmptyArray.isNonEmpty(ta.forest)) {
            return pipe(f(ta.value), F.map((value) => (forest) => ({
                value,
                forest
            })), F.ap(pipe(ta.forest, traverseF(out(f)))));
        }
        return pipe(fb, F.map((b) => make(b)));
    };
    return out;
};
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = (a) => make(a);
/**
 * @since 3.0.0
 */
export const unit = of(undefined);
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const getShow = (S) => {
    const show = (t) => {
        return readonlyArray.isEmpty(t.forest)
            ? `tree(${S.show(t.value)})`
            : `tree(${S.show(t.value)}, [${t.forest.map(show).join(', ')}])`;
    };
    return {
        show
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getEq = (E) => {
    const R = eq.fromEquals((second) => (first) => E.equals(second.value)(first.value) && SA.equals(second.forest)(first.forest));
    const SA = readonlyArray.getEq(R);
    return R;
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipLeftPar = /*#__PURE__*/ apply.zipLeftPar(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export const zipRightPar = /*#__PURE__*/ apply.zipRightPar(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Flattenable = {
    map,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    flatMap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    traverse
};
/**
 * @since 3.0.0
 */
export const sequence = (F) => (self) => pipe(self, traverse(F)(identity));
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const exists = (predicate) => (ma) => predicate(ma.value) || ma.forest.some(exists(predicate));
/**
 * Tests whether a value is a member of a `Tree`.
 *
 * @since 3.0.0
 */
export const elem = (E) => (a) => exists(E.equals(a));
const draw = (indentation, forest) => {
    let r = '';
    const len = forest.length;
    let tree;
    for (let i = 0; i < len; i++) {
        tree = forest[i];
        const isLast = i === len - 1;
        r += indentation + (isLast ? '└' : '├') + '─ ' + tree.value;
        r += draw(indentation + (len > 1 && !isLast ? '│  ' : '   '), tree.forest);
    }
    return r;
};
/**
 * Neat 2-dimensional drawing of a forest
 *
 * @since 3.0.0
 */
export const drawForest = (forest) => draw('\n', forest);
/**
 * Neat 2-dimensional drawing of a tree
 *
 * @example
 * import { make, drawTree } from 'fp-ts/Tree'
 *
 * const tree = make('a', [
 *   make('b'),
 *   make('c'),
 *   make('d', [make('e'), make('f')])
 * ])
 *
 * assert.strictEqual(drawTree(tree), `a
 * ├─ b
 * ├─ c
 * └─ d
 *    ├─ e
 *    └─ f`)
 *
 *
 * @since 3.0.0
 */
export const drawTree = (tree) => tree.value + drawForest(tree.forest);
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const bindTo = 
/*#__PURE__*/ functor.bindTo(Functor);
const let_ = 
/*#__PURE__*/ functor.let(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = 
/*#__PURE__*/ flattenable.bind(Flattenable);
/**
 * @since 3.0.0
 */
export const bindPar = 
/*#__PURE__*/ apply.bindPar(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ functor.tupled(Functor);
/**
 * @since 3.0.0
 */
export const apT = 
/*#__PURE__*/ apply.apT(Apply);
