import { identity, pipe } from './function';
import * as functor from './Functor';
import * as _ from './internal';
import * as readonlyNonEmptyArrayModule from './ReadonlyNonEmptyArray';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export const tell = (w) => [w, undefined];
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const fst = (self) => self[0];
/**
 * @since 3.0.0
 */
export const snd = (self) => self[1];
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export const evaluate = snd;
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export const execute = fst;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = (self) => [snd(self), fst(self)];
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export const listen = (self) => {
    const [w, a] = self;
    return [w, [w, a]];
};
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export const pass = (self) => {
    const [w, [a, f]] = self;
    return [f(w), a];
};
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export const listens = (f) => (self) => {
    const [w, a] = self;
    return [w, [a, f(w)]];
};
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export const censor = (f) => (self) => {
    const [w, a] = self;
    return [f(w), a];
};
// -------------------------------------------------------------------------------------
// type class operations
// -------------------------------------------------------------------------------------
/**
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (self) => {
    const [w, a] = self;
    return [w, f(a)];
};
/**
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapLeft = (f) => (self) => {
    const [w, a] = self;
    return [f(w), a];
};
/**
 * @category Bifunctor
 * @since 3.0.0
 */
export const mapBoth = (f, g) => (self) => [f(fst(self)), g(snd(self))];
/**
 * @category type class operations
 * @since 3.0.0
 */
export const compose = (bc) => (ab) => [fst(ab), snd(bc)];
/**
 * @category type class operations
 * @since 3.0.0
 */
export const extend = (f) => (self) => [fst(self), f(self)];
/**
 * @category type class operations
 * @since 3.0.0
 */
export const extract = snd;
/**
 * Derivable from `Extendable`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const duplicate = /*#__PURE__*/ extend(identity);
/**
 * @category type class operations
 * @since 3.0.0
 */
export const reduce = (b, f) => (self) => f(b, snd(self));
/**
 * @category type class operations
 * @since 3.0.0
 */
export const foldMap = (_M) => (f) => (self) => f(snd(self));
/**
 * @category type class operations
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (self) => f(snd(self), b);
/**
 * @category type class operations
 * @since 3.0.0
 */
export const traverse = (F) => (f) => (self) => pipe(f(snd(self)), F.map((b) => [fst(self), b]));
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    mapBoth
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = 
/*#__PURE__*/ functor.flap(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Composable = {
    compose
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map,
    extend,
    extract
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    traverse
};
/**
 * @since 3.0.0
 */
export const sequence = (F) => traverse(F)(identity);
/**
 * @category instances
 * @since 3.0.0
 */
export const getPointed = (M) => ({
    of: (a) => [M.empty, a]
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map,
    ap: (fa) => (fab) => {
        const [w1, f] = fab;
        const [w2, a] = fa;
        return [S.combine(w2)(w1), f(a)];
    }
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (M) => {
    const A = getApply(M);
    const P = getPointed(M);
    return {
        map,
        ap: A.ap,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getFlattenable = (S) => {
    return {
        map,
        flatMap: (f) => (ma) => {
            const [w1, a] = ma;
            const [w2, b] = f(a);
            return [S.combine(w2)(w1), b];
        }
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (M) => {
    const P = getPointed(M);
    const C = getFlattenable(M);
    return {
        map,
        of: P.of,
        flatMap: C.flatMap
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export function getFlattenableRec(M) {
    const flatMapRec = (f) => (a) => {
        let result = f(a);
        let acc = M.empty;
        let s = snd(result);
        while (_.isLeft(s)) {
            acc = M.combine(fst(result))(acc);
            result = f(s.left);
            s = snd(result);
        }
        return [M.combine(fst(result))(acc), s.right];
    };
    return {
        flatMapRec
    };
}
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (S) => (f) => (as) => {
    // TODO
    return readonlyNonEmptyArrayModule.traverseWithIndex(getApply(S))(f)(as);
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (M) => (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(M)(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : [M.empty, _.emptyReadonlyArray]);
};
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(S))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArray = (S) => {
    const traverseReadonlyNonEmptyArrayWithIndexS = traverseReadonlyNonEmptyArrayWithIndex(S);
    return (f) => {
        return traverseReadonlyNonEmptyArrayWithIndexS((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArray = (M) => {
    const traverseReadonlyArrayWithIndexS = traverseReadonlyArrayWithIndex(M);
    return (f) => {
        return traverseReadonlyArrayWithIndexS((_, a) => f(a));
    };
};
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export const sequenceReadonlyArray = (M) => traverseReadonlyArray(M)(identity);
