import * as eq from './Eq';
// -------------------------------------------------------------------------------------
// refinements
// -------------------------------------------------------------------------------------
/**
 * @category refinements
 * @since 3.0.0
 */
export const isNumber = (u) => typeof u === 'number';
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Eq = eq.EqStrict;
/**
 * @category instances
 * @since 3.0.0
 */
export const Ord = {
    compare: (second) => (first) => first < second ? -1 : first > second ? 1 : 0
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Bounded = {
    compare: Ord.compare,
    top: Infinity,
    bottom: -Infinity
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Field = {
    add: (second) => (first) => first + second,
    zero: 0,
    mul: (second) => (first) => first * second,
    one: 1,
    sub: (second) => (first) => first - second,
    degree: () => 1,
    div: (second) => (first) => first / second,
    mod: (second) => (first) => first % second
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Show = {
    show: (a) => JSON.stringify(a)
};
/**
 * @category instances
 * @since 3.0.0
 */
export const MagmaSub = {
    combine: Field.sub
};
/**
 * `number` semigroup under addition.
 *
 * @example
 * import { SemigroupSum } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupSum.combine(3)), 5)
 *
 * @category instances
 * @since 3.0.0
 */
export const SemigroupSum = {
    combine: (second) => (first) => first + second
};
/**
 * `number` semigroup under multiplication.
 *
 * @example
 * import { SemigroupProduct } from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(2, SemigroupProduct.combine(3)), 6)
 *
 * @category instances
 * @since 3.0.0
 */
export const SemigroupProduct = {
    combine: (second) => (first) => first * second
};
/**
 * `number` monoid under addition.
 *
 * The `empty` value is `0`.
 *
 * @category instances
 * @since 3.0.0
 */
export const MonoidSum = {
    combine: SemigroupSum.combine,
    empty: 0
};
/**
 * `number` monoid under multiplication.
 *
 * The `empty` value is `1`.
 *
 * @category instances
 * @since 3.0.0
 */
export const MonoidProduct = {
    combine: SemigroupProduct.combine,
    empty: 1
};
