/**
 * The `Const` type constructor, which wraps its first type argument and ignores its second.
 * That is, `Const<E, A>` is isomorphic to `E` for any `A`.
 *
 * `Const` has some useful instances. For example, the `Applicative` instance allows us to collect results using a `Monoid`
 * while ignoring return values.
 *
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type { Apply } from './Apply'
import type * as bifunctor from './Bifunctor'
import type { BooleanAlgebra } from './BooleanAlgebra'
import type { Bounded } from './Bounded'
import type * as contravariant from './Contravariant'
import type { Eq } from './Eq'
import * as functor from './Functor'
import type { HeytingAlgebra } from './HeytingAlgebra'
import type { TypeLambda } from './HKT'
import type { Monoid } from './Monoid'
import type { Ord } from './Ord'
import type { Ring } from './Ring'
import type { Semigroup } from './Semigroup'
import type { Semiring } from './Semiring'
import type { Show } from './Show'
/**
 * @since 3.0.0
 */
export declare const phantom: unique symbol
/**
 * @category model
 * @since 3.0.0
 */
export interface Const</** in out */ S, /** out */ A> {
  readonly [phantom]: A
  readonly value: S
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Constλ extends TypeLambda {
  readonly type: Const<this['InOut1'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ConstλBifunctor extends TypeLambda {
  readonly type: Const<this['Out2'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ConstλContravariant extends TypeLambda {
  readonly type: Const<this['InOut1'], this['In1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ConstλFix<S> extends TypeLambda {
  readonly type: Const<S, this['Out1']>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const make: <S>(s: S) => Const<S, never>
/**
 * @since 3.0.0
 */
export declare const contramap: <B, A>(f: (b: B) => A) => <S>(fa: Const<S, A>) => Const<S, B>
/**
 * @since 3.0.0
 */
export declare const mapLeft: <S, G>(f: (s: S) => G) => <A>(self: Const<S, A>) => Const<G, A>
/**
 * @since 3.0.0
 */
export declare const mapBoth: <S, T, A, B>(f: (s: S) => T, g: (a: A) => B) => (self: Const<S, A>) => Const<T, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <S>(E: Eq<S>) => Eq<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getOrd: <S>(O: Ord<S>) => Ord<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getBounded: <S>(B: Bounded<S>) => Bounded<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <S>(S: Show<S>) => Show<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getSemigroup: <S>(S: Semigroup<S>) => Semigroup<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <S>(M: Monoid<S>) => Monoid<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getSemiring: <S>(S: Semiring<S>) => Semiring<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getRing: <S>(R: Ring<S>) => Ring<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getHeytingAlgebra: <S>(H: HeytingAlgebra<S>) => HeytingAlgebra<Const<S, never>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getBooleanAlgebra: <S>(H: BooleanAlgebra<S>) => BooleanAlgebra<Const<S, never>>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <S>(self: Const<S, A>) => Const<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Constλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <S, B>(self: Const<S, (a: A) => B>) => Const<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Contravariant: contravariant.Contravariant<ConstλContravariant>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: bifunctor.Bifunctor<ConstλBifunctor>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <S>(S: Semigroup<S>) => Apply<ConstλFix<S>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <S>(M: Monoid<S>) => Applicative<ConstλFix<S>>
