/**
 * The `FromEither` type class represents those data types which support typed errors.
 *
 * @since 3.0.0
 */
import type { Flattenable } from './Flattenable'
import type { Either } from './Either'
import type { LazyArg } from './function'
import type { TypeLambda, Kind, TypeClass } from './HKT'
import type { Option } from './Option'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromEither<F extends TypeLambda> extends TypeClass<F> {
  readonly fromEither: <E, A, S>(fa: Either<E, A>) => Kind<F, S, unknown, never, E, A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromOption: <F extends TypeLambda>(
  F: FromEither<F>
) => <E>(onNone: LazyArg<E>) => <A, S>(fa: Option<A>) => Kind<F, S, unknown, never, E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: <F extends TypeLambda>(
  F: FromEither<F>
) => {
  <C extends A, B extends A, E, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E): <S>(
    c: C
  ) => Kind<F, S, unknown, never, E, B>
  <B extends A, E, A = B>(predicate: Predicate<A>, onFalse: (b: B) => E): <S>(b: B) => Kind<F, S, unknown, never, E, B>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <F extends TypeLambda>(
  F: FromEither<F>
) => <A extends readonly unknown[], B, E>(
  f: (...a: A) => Option<B>,
  onNone: (...a: A) => E
) => <S>(...a: A) => Kind<F, S, unknown, never, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapOptionK: <M extends TypeLambda>(
  F: FromEither<M>,
  M: Flattenable<M>
) => <A, B, E>(
  f: (a: A) => Option<B>,
  onNone: (a: A) => E
) => <S, R, W>(self: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <F extends TypeLambda>(
  F: FromEither<F>
) => <A extends readonly unknown[], E, B>(
  f: (...a: A) => Either<E, B>
) => <S>(...a: A) => Kind<F, S, unknown, never, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapEitherK: <M extends TypeLambda>(
  F: FromEither<M>,
  M: Flattenable<M>
) => <A, E2, B>(
  f: (a: A) => Either<E2, B>
) => <S, R, W, E1>(self: Kind<M, S, R, W, E1, A>) => Kind<M, S, R, W, E2 | E1, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const filterMap: <F extends TypeLambda>(
  F: FromEither<F>,
  M: Flattenable<F>
) => <A, B, E>(
  f: (a: A) => Option<B>,
  onNone: (a: A) => E
) => <S, R, W>(self: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const partitionMap: <F extends TypeLambda>(
  F: FromEither<F>,
  M: Flattenable<F>
) => <A, B, C, E>(
  f: (a: A) => Either<B, C>,
  onEmpty: (a: A) => E
) => <S, R, W>(self: Kind<F, S, R, W, E, A>) => readonly [Kind<F, S, R, W, E, B>, Kind<F, S, R, W, E, C>]
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const filter: <M extends TypeLambda>(
  F: FromEither<M>,
  M: Flattenable<M>
) => {
  <C extends A, B extends A, E2, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E2): <S, R, W, E1>(
    self: Kind<M, S, R, W, E1, C>
  ) => Kind<M, S, R, W, E2 | E1, B>
  <B_1 extends A_1, E2_1, A_1 = B_1>(predicate: Predicate<A_1>, onFalse: (b: B_1) => E2_1): <S_1, R_1, W_1, E1_1>(
    self: Kind<M, S_1, R_1, W_1, E1_1, B_1>
  ) => Kind<M, S_1, R_1, W_1, E2_1 | E1_1, B_1>
}
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const partition: <F extends TypeLambda>(
  F: FromEither<F>,
  M: Flattenable<F>
) => {
  <C extends A, B extends A, E, A = C>(refinement: Refinement<A, B>, onFalse: (c: C) => E): <S, R, W>(
    self: Kind<F, S, R, W, E, C>
  ) => readonly [Kind<F, S, R, W, E, C>, Kind<F, S, R, W, E, B>]
  <B_1 extends A_1, E_1, A_1 = B_1>(predicate: Predicate<A_1>, onFalse: (b: B_1) => E_1): <S_1, R_1, W_1>(
    self: Kind<F, S_1, R_1, W_1, E_1, B_1>
  ) => readonly [Kind<F, S_1, R_1, W_1, E_1, B_1>, Kind<F, S_1, R_1, W_1, E_1, B_1>]
}
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <F extends TypeLambda>(
  F: FromEither<F>
) => <E>(onNullable: LazyArg<E>) => <A, S>(a: A) => Kind<F, S, unknown, never, E, NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <F extends TypeLambda>(
  F: FromEither<F>
) => <E>(
  onNullable: LazyArg<E>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => B | null | undefined
) => <S>(...a: A) => Kind<F, S, unknown, never, E, NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const flatMapNullableK: <M extends TypeLambda>(
  F: FromEither<M>,
  M: Flattenable<M>
) => <E>(
  onNullable: LazyArg<E>
) => <A, B>(
  f: (a: A) => B | null | undefined
) => <S, R, W>(self: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, NonNullable<B>>
