/**
 * `IOOption<A>` represents a synchronous computation that either yields a value of type `A` or nothing.
 *
 * If you want to represent a synchronous computation that never fails, please see `IO`.
 * If you want to represent a synchronous computation that may fail, please see `IOEither`.
 *
 * @since 3.0.0
 */
import type * as semigroupK from './SemigroupK'
import * as monoidK from './MonoidK'
import type * as applicative from './Applicative'
import * as apply from './Apply'
import * as flattenable from './Flattenable'
import * as compactable from './Compactable'
import type { Either } from './Either'
import * as filterable from './Filterable'
import * as fromOption_ from './FromOption'
import * as fromEither_ from './FromEither'
import * as fromIO_ from './FromIO'
import type { LazyArg } from './function'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type { IOEither } from './IOEither'
import type * as monad from './Monad'
import * as option from './Option'
import type * as pointed from './Pointed'
import type { Predicate } from './Predicate'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Refinement } from './Refinement'
import type { IO } from './IO'
import type { Option } from './Option'
/**
 * @category model
 * @since 3.0.0
 */
export interface IOOption<A> extends IO<Option<A>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const some: <A>(a: A) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <A>(fa: Option<A>) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromEither: <A>(e: Either<unknown, A>) => IO<option.Option<A>>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIO: <A>(ma: IO<A>) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: <A>(ma: IOEither<unknown, A>) => IOOption<A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <B, A, C = B>(onNone: LazyArg<B>, onSome: (a: A) => C) => (ma: IOOption<A>) => IO<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchWithEffect: <B, A, C = B>(
  onNone: LazyArg<IO<B>>,
  onSome: (a: A) => IO<C>
) => (ma: IOOption<A>) => IO<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElse: <B>(onNone: LazyArg<B>) => <A>(ma: IOOption<A>) => IO<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElseE: <B>(onNone: LazyArg<IO<B>>) => <A>(ma: IOOption<A>) => IO<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const toUndefined: <A>(ma: IOOption<A>) => IO<A | undefined>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const toNullable: <A>(ma: IOOption<A>) => IO<A | null>
/**
 * Returns an effect whose success is mapped by the specified `f` function.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: IOOption<A>) => IOOption<B>
/**
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: IOOption<A>) => <B>(fab: IOOption<(a: A) => B>) => IOOption<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => IOOption<A>
/**
 * @since 3.0.0
 */
export declare const unit: IOOption<void>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export declare const flatMap: <A, B>(f: (a: A) => IOOption<B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: IOOption<IOOption<A>>) => IOOption<A>
/**
 * @category SemigroupK
 * @since 3.0.0
 */
export declare const combineK: <B>(second: LazyArg<IOOption<B>>) => <A>(self: IOOption<A>) => IOOption<A | B>
/**
 * @category MonoidK
 * @since 3.0.0
 */
export declare const emptyK: <A>() => IOOption<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const none: IOOption<never>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <A>(foa: IOOption<option.Option<A>>) => IOOption<A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <A, B>(fe: IOOption<Either<A, B>>) => readonly [IOOption<A>, IOOption<B>]
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => Option<B>) => (fga: IOOption<A>) => IOOption<B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => (fa: IOOption<A>) => readonly [IOOption<B>, IOOption<C>]
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface IOOptionλ extends TypeLambda {
  readonly type: IOOption<this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<IOOptionλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: IOOption<(a: A) => B>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<IOOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: apply.Apply<IOOptionλ>
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, this effect result returned. If either side fails, then the
 * other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <B>(second: IOOption<B>) => <A>(self: IOOption<A>) => IOOption<A>
/**
 * Returns an effect that executes both this effect and the specified effect,
 * in parallel, returning result of provided effect. If either side fails,
 * then the other side will **NOT** be interrupted.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <B>(second: IOOption<B>) => <A>(self: IOOption<A>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: applicative.Applicative<IOOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Flattenable: flattenable.Flattenable<IOOptionλ>
/**
 * Returns an effect that effectfully "peeks" at the success of this effect.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const tap: <A, _>(f: (a: A) => IOOption<_>) => (self: IOOption<A>) => IOOption<A>
/**
 * Returns an effect that effectfully "peeks" at the failure of this effect.
 *
 * @category combinatorsError
 * @since 3.0.0
 */
export declare const tapError: <_>(onNone: () => IOOption<_>) => <A>(self: IOOption<A>) => IOOption<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupK: semigroupK.SemigroupK<IOOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidK: monoidK.MonoidK<IOOptionλ>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const guard: (b: boolean) => IOOption<void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: monad.Monad<IOOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: compactable.Compactable<IOOptionλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: filterable.Filterable<IOOptionλ>
/**
 * @since 3.0.0
 */
export declare const filter: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): (fc: IOOption<C>) => IOOption<B>
  <B extends A, A = B>(predicate: Predicate<A>): (fb: IOOption<B>) => IOOption<B>
}
/**
 * @since 3.0.0
 */
export declare const partition: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): (
    fc: IOOption<C>
  ) => readonly [IOOption<C>, IOOption<B>]
  <B extends A, A = B>(predicate: Predicate<A>): (fb: IOOption<B>) => readonly [IOOption<B>, IOOption<B>]
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: fromIO_.FromIO<IOOptionλ>
/**
 * @category logging
 * @since 3.0.0
 */
export declare const log: (...x: ReadonlyArray<unknown>) => IOOption<void>
/**
 * @category logging
 * @since 3.0.0
 */
export declare const logError: (...x: ReadonlyArray<unknown>) => IOOption<void>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => IO<B>) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapIOK: <A, B>(f: (a: A) => IO<B>) => (self: IOOption<A>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromOption: fromOption_.FromOption<IOOptionλ>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: <B extends A, A = B>(predicate: Predicate<A>) => (b: B) => IOOption<B>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromRefinement: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => (c: C) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => Option<B>
) => (...a: A) => IOOption<B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <A>(a: A) => IOOption<NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => IOOption<NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const flatMapNullableK: <A, B>(
  f: (a: A) => B | null | undefined
) => (ma: IOOption<A>) => IOOption<NonNullable<B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: fromEither_.FromEither<IOOptionλ>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const flatMapEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * @since 3.0.0
 */
export declare const Do: IOOption<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(fa: IOOption<A>) => IOOption<{
  readonly [K in N]: A
}>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (fa: IOOption<A>) => IOOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => IOOption<B>
) => (ma: IOOption<A>) => IOOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const bindPar: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: IOOption<B>
) => (fa: IOOption<A>) => IOOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const ApT: IOOption<readonly []>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IOOption<B>
) => (as: readonly [A, ...A[]]) => IOOption<readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IOOption<B>
) => (as: readonly A[]) => IOOption<readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, B>(
  f: (a: A) => IOOption<B>
) => (as: readonly [A, ...A[]]) => IOOption<readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, B>(
  f: (a: A) => IOOption<B>
) => (as: readonly A[]) => IOOption<readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <A>(arr: ReadonlyArray<IOOption<A>>) => IOOption<ReadonlyArray<A>>
