/**
 * @since 3.0.0
 */
import type * as applicative from './Applicative'
import * as apply from './Apply'
import type * as category from './Category'
import type * as composable from './Composable'
import * as flattenable from './Flattenable'
import type * as fromReader_ from './FromReader'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type * as monad from './Monad'
import type * as pointed from './Pointed'
import type * as profunctor from './Profunctor'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface Reader<R, A> {
  (r: R): A
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Readerλ extends TypeLambda {
  readonly type: Reader<this['In1'], this['Out1']>
}
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const ask: <R>() => Reader<R, R>
/**
 * Projects a value from the global context in a `Reader`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const asks: <R, A>(f: (r: R) => A) => Reader<R, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReader: <R1, R2, A>(f: (r1: R1) => Reader<R2, A>) => Reader<R1 & R2, A>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(f: (r2: R2) => R1) => <A>(ma: Reader<R1, A>) => Reader<R2, A>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <R>(fa: Reader<R, A>) => Reader<R, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <R2, A>(fa: Reader<R2, A>) => <R1, B>(fab: Reader<R1, (a: A) => B>) => Reader<R1 & R2, B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => Reader<unknown, A>
/**
 * @since 3.0.0
 */
export declare const unit: Reader<unknown, void>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export declare const flatMap: <A, R2, B>(f: (a: A) => Reader<R2, B>) => <R1>(ma: Reader<R1, A>) => Reader<R1 & R2, B>
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <R1, R2, A>(mma: Reader<R1, Reader<R2, A>>) => Reader<R1 & R2, A>
/**
 * @category Composable
 * @since 3.0.0
 */
export declare const compose: <B, C>(bc: Reader<B, C>) => <A>(ab: Reader<A, B>) => Reader<A, C>
/**
 * @category Profunctor
 * @since 3.0.0
 */
export declare const promap: <Q, R, A, B>(f: (d: Q) => R, g: (a: A) => B) => (pea: Reader<R, A>) => Reader<Q, B>
/**
 * @category Category
 * @since 3.0.0
 */
export declare const id: <A>() => Reader<A, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromReader: fromReader_.FromReader<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Readerλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <R, B>(fab: Reader<R, (a: A) => B>) => Reader<R, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: apply.Apply<Readerλ>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <R2, B>(second: Reader<R2, B>) => <R1, A>(self: Reader<R1, A>) => Reader<R1 & R2, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <R2, B>(second: Reader<R2, B>) => <R1, A>(self: Reader<R1, A>) => Reader<R1 & R2, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: applicative.Applicative<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Flattenable: flattenable.Flattenable<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: monad.Monad<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Profunctor: profunctor.Profunctor<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Composable: composable.Composable<Readerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Category: category.Category<Readerλ>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <R, A>(
  fa: Reader<R, A>
) => Reader<
  R,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <R>(fa: Reader<R, A>) => Reader<
  R,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => Reader<R2, B>
) => <R1>(fa: Reader<R1, A>) => Reader<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const Do: Reader<unknown, {}>
/**
 * @since 3.0.0
 */
export declare const bindPar: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  fb: Reader<R2, B>
) => <R1>(fa: Reader<R1, A>) => Reader<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: Reader<unknown, readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <R, A>(fa: Reader<R, A>) => Reader<R, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <R2, B>(
  fb: Reader<R2, B>
) => <R1, A extends ReadonlyArray<unknown>>(fas: Reader<R1, A>) => Reader<R1 & R2, readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => Reader<R, B>
) => (as: readonly [A, ...A[]]) => Reader<R, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => Reader<R, B>
) => (as: readonly A[]) => Reader<R, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, R, B>(
  f: (a: A) => Reader<R, B>
) => (as: readonly [A, ...A[]]) => Reader<R, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, R, B>(
  f: (a: A) => Reader<R, B>
) => (as: readonly A[]) => Reader<R, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <R, A>(arr: ReadonlyArray<Reader<R, A>>) => Reader<R, ReadonlyArray<A>>
