/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type * as compactable from './Compactable'
import type { Either } from './Either'
import type { Endomorphism } from './Endomorphism'
import * as eq from './Eq'
import * as filterable from './Filterable'
import * as filterableWithIndex from './FilterableWithIndex'
import type { Foldable } from './Foldable'
import type { FoldableWithIndex } from './FoldableWithIndex'
import * as functor from './Functor'
import type { FunctorWithIndex } from './FunctorWithIndex'
import type { TypeLambda, Kind } from './HKT'
import type { Magma } from './Magma'
import type { Monoid } from './Monoid'
import * as option from './Option'
import type { Ord } from './Ord'
import type { Semigroup } from './Semigroup'
import type { Show } from './Show'
import type { Traversable } from './Traversable'
import type { TraversableWithIndex } from './TraversableWithIndex'
import type { Unfoldable } from './Unfoldable'
import * as filterableWithEffect from './FilterableWithEffect'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import type { Eq } from './Eq'
import type { Option } from './Option'
/**
 * Create a `ReadonlyMap` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const singleton: <K, A>(k: K, a: A) => ReadonlyMap<K, A>
/**
 * Create a `ReadonlyRecord` from a `Foldable` collection of key/value pairs, using the
 * specified `Magma` to combine values for duplicate keys, and the specified `f` to map to key/value pairs.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function fromFoldable<F extends TypeLambda>(
  F: Foldable<F>
): <K, B>(
  E: Eq<K>,
  M: Magma<B>
) => <A>(f: (a: A) => readonly [K, B]) => <S, R, W, E>(fka: Kind<F, S, R, W, E, A>) => ReadonlyMap<K, B>
/**
 * Insert an element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const insertAt: <K>(
  E: eq.Eq<K>
) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => option.Option<ReadonlyMap<K, A>>
/**
 * Insert or replace a key/value pair in a `ReadonlyMap`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const upsertAt: <K>(E: eq.Eq<K>) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * Change the element at the specified keys, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export declare const updateAt: <K>(
  E: eq.Eq<K>
) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => option.Option<ReadonlyMap<K, A>>
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export declare const modifyAt: <K>(
  E: eq.Eq<K>
) => <A>(k: K, f: Endomorphism<A>) => (m: ReadonlyMap<K, A>) => option.Option<ReadonlyMap<K, A>>
/**
 * Delete the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const deleteAt: <K>(
  E: eq.Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => option.Option<ReadonlyMap<K, A>>
/**
 * Delete a key and value from a `ReadonlyMap`, returning the value as well as the subsequent `ReadonlyMap`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pop: <K>(
  E: eq.Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => option.Option<readonly [A, ReadonlyMap<K, A>]>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <K>(fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export declare const mapWithIndex: <K, A, B>(f: (k: K, a: A) => B) => (m: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <K, A>(m: ReadonlyMap<K, option.Option<A>>) => ReadonlyMap<K, A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <K, A, B>(
  fa: ReadonlyMap<K, Either<A, B>>
) => readonly [ReadonlyMap<K, A>, ReadonlyMap<K, B>]
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => option.Option<B>) => <K>(fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => <K>(fa: ReadonlyMap<K, A>) => readonly [ReadonlyMap<K, B>, ReadonlyMap<K, C>]
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare const filterMapWithIndex: <K, A, B>(
  f: (k: K, a: A) => option.Option<B>
) => (fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare const partitionMapWithIndex: <K, A, B, C>(
  f: (k: K, a: A) => Either<B, C>
) => (fa: ReadonlyMap<K, A>) => readonly [ReadonlyMap<K, B>, ReadonlyMap<K, C>]
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ReadonlyMapλ extends TypeLambda {
  readonly type: ReadonlyMap<this['InOut1'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ReadonlyMapλFix<K> extends TypeLambda {
  readonly type: ReadonlyMap<K, this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <K, A>(SK: Show<K>, SA: Show<A>) => Show<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <K, A>(EK: eq.Eq<K>, EA: eq.Eq<A>) => eq.Eq<ReadonlyMap<K, A>>
/**
 * Get a `Monoid` instance for `ReadonlyMap` given a `Semigroup` instance for its values.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <K, A>(EK: eq.Eq<K>, SA: Semigroup<A>) => Monoid<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<ReadonlyMapλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <K, B>(fab: ReadonlyMap<K, (a: A) => B>) => ReadonlyMap<K, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFunctorWithIndex: <K = never>() => FunctorWithIndex<ReadonlyMapλFix<K>, K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: compactable.Compactable<ReadonlyMapλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: filterable.Filterable<ReadonlyMapλ>
/**
 * @since 3.0.0
 */
export declare const filter: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): <K>(fc: ReadonlyMap<K, C>) => ReadonlyMap<K, B>
  <B extends A, A = B>(predicate: Predicate<A>): <K>(fb: ReadonlyMap<K, B>) => ReadonlyMap<K, B>
}
/**
 * @since 3.0.0
 */
export declare const partition: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): <K>(
    fc: ReadonlyMap<K, C>
  ) => readonly [ReadonlyMap<K, C>, ReadonlyMap<K, B>]
  <B extends A, A = B>(predicate: Predicate<A>): <K>(
    fb: ReadonlyMap<K, B>
  ) => readonly [ReadonlyMap<K, B>, ReadonlyMap<K, B>]
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFilterableWithIndex: <K = never>() => filterableWithIndex.FilterableWithIndex<
  ReadonlyMapλFix<K>,
  K
>
/**
 * @since 3.0.0
 */
export declare const filterWithIndex: {
  <K, C extends A, B extends A, A = C>(refinement: (i: K, a: A) => a is B): (fc: ReadonlyMap<K, C>) => ReadonlyMap<K, B>
  <K, B extends A, A = B>(predicate: (i: K, a: A) => boolean): (fb: ReadonlyMap<K, B>) => ReadonlyMap<K, B>
}
/**
 * @since 3.0.0
 */
export declare const partitionWithIndex: {
  <K, C extends A, B extends A, A = C>(refinement: (i: K, a: A) => a is B): (
    fb: ReadonlyMap<K, C>
  ) => readonly [ReadonlyMap<K, C>, ReadonlyMap<K, B>]
  <K, B extends A, A = B>(predicate: (i: K, a: A) => boolean): (
    fb: ReadonlyMap<K, B>
  ) => readonly [ReadonlyMap<K, B>, ReadonlyMap<K, B>]
}
/**
 * @since 3.0.0
 */
export declare const reduce: <K>(O: Ord<K>) => <B, A>(b: B, f: (b: B, a: A) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: <K>(O: Ord<K>) => <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: ReadonlyMap<K, A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRight: <K>(O: Ord<K>) => <B, A>(b: B, f: (a: A, b: B) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldable: <K>(O: Ord<K>) => Foldable<ReadonlyMapλFix<K>>
/**
 * @since 3.0.0
 */
export declare const reduceWithIndex: <K>(
  O: Ord<K>
) => <B, A>(b: B, f: (i: K, b: B, a: A) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMapWithIndex: <K>(
  O: Ord<K>
) => <M>(M: Monoid<M>) => <A>(f: (i: K, a: A) => M) => (fa: ReadonlyMap<K, A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRightWithIndex: <K>(
  O: Ord<K>
) => <B, A>(b: B, f: (i: K, a: A, b: B) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldableWithIndex: <K>(O: Ord<K>) => FoldableWithIndex<ReadonlyMapλFix<K>, K>
/**
 * @since 3.0.0
 */
export declare const traverse: <K>(
  O: Ord<K>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<F, S, R, W, E, B>
) => (ta: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const sequence: <K>(
  O: Ord<K>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <S, R, W, E, A>(ta: ReadonlyMap<K, Kind<F, S, R, W, E, A>>) => Kind<F, S, R, W, E, ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversable: <K>(O: Ord<K>) => Traversable<ReadonlyMapλFix<K>>
/**
 * @since 3.0.0
 */
export declare const traverseWithIndex: <K>(
  O: Ord<K>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (i: K, a: A) => Kind<F, S, R, W, E, B>
) => (ta: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, ReadonlyMap<K, B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversableWithIndex: <K>(O: Ord<K>) => TraversableWithIndex<ReadonlyMapλFix<K>, K>
/**
 * @since 3.0.0
 */
export declare const getFilterMapE: <K>(
  O: Ord<K>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<F, S, R, W, E, option.Option<B>>
) => (ta: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const getPartitionMapE: <K>(
  O: Ord<K>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B, C>(
  f: (a: A) => Kind<F, S, R, W, E, Either<B, C>>
) => (wa: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, readonly [ReadonlyMap<K, B>, ReadonlyMap<K, C>]>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFilterableWithEffect: <K>(
  O: Ord<K>
) => filterableWithEffect.FilterableWithEffect<ReadonlyMapλFix<K>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionSemigroup: <K, A>(E: eq.Eq<K>, S: Semigroup<A>) => Semigroup<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionMonoid: <K, A>(E: eq.Eq<K>, S: Semigroup<A>) => Monoid<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getIntersectionSemigroup: <K, A>(E: eq.Eq<K>, S: Semigroup<A>) => Semigroup<ReadonlyMap<K, A>>
/**
 * @category combinator
 * @since 3.0.0
 */
export declare const getDifferenceMagma: <K>(E: eq.Eq<K>) => <A>() => Magma<ReadonlyMap<K, A>>
/**
 * Calculate the number of key/value pairs in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const size: <K, A>(d: ReadonlyMap<K, A>) => number
/**
 * Test whether or not a `ReadonlyMap` is empty.
 *
 * @since 3.0.0
 */
export declare const isEmpty: <K, A>(d: ReadonlyMap<K, A>) => boolean
/**
 * Test whether or not a key exists in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const member: <K>(E: eq.Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => boolean
/**
 * Tests whether a value is a member of a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: eq.Eq<A>) => (a: A) => <K>(m: ReadonlyMap<K, A>) => boolean
/**
 * Get a sorted `ReadonlyArray` of the keys contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const keys: <K>(O: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => readonly K[]
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const values: <A>(O: Ord<A>) => <K>(m: ReadonlyMap<K, A>) => readonly A[]
/**
 * @since 3.0.0
 */
export declare const collect: <K>(O: Ord<K>) => <A, B>(f: (k: K, a: A) => B) => (m: ReadonlyMap<K, A>) => readonly B[]
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 * If the result is a `Some`, the existing key is also returned.
 *
 * @since 3.0.0
 */
export declare const lookupWithKey: <K>(
  E: eq.Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => option.Option<readonly [K, A]>
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const lookup: <K>(E: eq.Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => option.Option<A>
/**
 * Test whether or not one `ReadonlyMap` contains all of the keys and values contained in another `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const isSubmap: <K, A>(
  EK: eq.Eq<K>,
  SA: eq.Eq<A>
) => (second: ReadonlyMap<K, A>) => (self: ReadonlyMap<K, A>) => boolean
/**
 * @since 3.0.0
 */
export declare const emptyK: <K>() => ReadonlyMap<K, never>
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const toReadonlyArray: <K>(O: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => readonly (readonly [K, A])[]
/**
 * Unfolds a `ReadonlyMap` into a data structure of key/value pairs.
 *
 * @since 3.0.0
 */
export declare function toUnfoldable<F extends TypeLambda>(
  U: Unfoldable<F>
): <K>(o: Ord<K>) => <A, S, R, W, E>(d: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, readonly [K, A]>
/**
 * @since 3.0.0
 */
export declare const union: <K, A>(
  E: eq.Eq<K>,
  M: Magma<A>
) => (second: ReadonlyMap<K, A>) => (self: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @since 3.0.0
 */
export declare const intersection: <K, A>(
  E: eq.Eq<K>,
  M: Magma<A>
) => (second: ReadonlyMap<K, A>) => (self: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @since 3.0.0
 */
export declare const difference: <K>(
  E: eq.Eq<K>
) => <A>(_second: ReadonlyMap<K, A>) => (self: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
