/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import type * as compactable from './Compactable'
import type { Either } from './Either'
import type { Endomorphism } from './Endomorphism'
import * as eq from './Eq'
import * as filterable from './Filterable'
import * as filterableWithIndex from './FilterableWithIndex'
import type { Foldable } from './Foldable'
import type * as foldableWithIndex from './FoldableWithIndex'
import * as functor from './Functor'
import type * as functorWithIndex from './FunctorWithIndex'
import type { TypeLambda, Kind } from './HKT'
import type { Magma } from './Magma'
import type { Monoid } from './Monoid'
import * as option from './Option'
import type { Ord } from './Ord'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import type { Semigroup } from './Semigroup'
import type { Show } from './Show'
import type * as traversable from './Traversable'
import type * as traversableWithIndex from './TraversableWithIndex'
import type { Unfoldable } from './Unfoldable'
import * as filterableWithEffect from './FilterableWithEffect'
import type { Eq } from './Eq'
import type { Option } from './Option'
/**
 * @category model
 * @since 3.0.0
 */
export declare type ReadonlyRecord<K extends string, T> = Readonly<Record<K, T>>
/**
 * Create a `ReadonlyRecord` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const singleton: <A>(k: string, a: A) => Readonly<Record<string, A>>
/**
 * Create a `ReadonlyRecord` from a `Foldable` collection of key/value pairs, using the
 * specified `Magma` to combine values for duplicate keys, and the specified `f` to map to key/value pairs.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function fromFoldable<F extends TypeLambda>(
  F: Foldable<F>
): <B>(
  M: Magma<B>
) => <A>(f: (a: A) => readonly [string, B]) => <S, R, W, E>(r: Kind<F, S, R, W, E, A>) => ReadonlyRecord<string, B>
/**
 * Insert an element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const insertAt: <A>(
  k: string,
  a: A
) => (r: Readonly<Record<string, A>>) => option.Option<Readonly<Record<string, A>>>
/**
 * Insert or replace a key/value pair in a `ReadonlyRecord`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const upsertAt: <A>(k: string, a: A) => (r: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * Change the element at the specified keys, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const updateAt: <A>(
  k: string,
  a: A
) => (r: Readonly<Record<string, A>>) => option.Option<Readonly<Record<string, A>>>
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const modifyAt: <A>(
  k: string,
  f: Endomorphism<A>
) => (r: Readonly<Record<string, A>>) => option.Option<Readonly<Record<string, A>>>
/**
 * Delete the element at the specified key, creating a new `ReadonlyRecord`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const deleteAt: (
  k: string
) => <A>(r: Readonly<Record<string, A>>) => option.Option<Readonly<Record<string, A>>>
/**
 * Delete the element at the specified key, returning the value as well as the subsequent `ReadonlyRecord`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pop: (
  k: string
) => <A>(r: Readonly<Record<string, A>>) => option.Option<readonly [A, Readonly<Record<string, A>>]>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(
  f: (a: A) => B
) => <K extends string>(r: Readonly<Record<K, A>>) => Readonly<Record<K, B>>
/**
 * Map a `ReadonlyRecord` passing both the keys and values to the iterating function.
 *
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export declare function mapWithIndex<K extends string, A, B>(
  f: (k: K, a: A) => B
): (r: ReadonlyRecord<K, A>) => ReadonlyRecord<K, B>
/**
 * @since 3.0.0
 */
export declare const keys: (O: Ord<string>) => <K extends string>(r: Readonly<Record<K, unknown>>) => readonly K[]
/**
 * @since 3.0.0
 */
export declare const reduceWithIndex: (
  O: Ord<string>
) => <B, K extends string, A>(b: B, f: (k: K, b: B, a: A) => B) => (r: Readonly<Record<K, A>>) => B
/**
 * @since 3.0.0
 */
export declare const foldMapWithIndex: (
  O: Ord<string>
) => <M>(M: Monoid<M>) => <K extends string, A>(f: (k: K, a: A) => M) => (r: Readonly<Record<K, A>>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRightWithIndex: (
  O: Ord<string>
) => <B, K extends string, A>(b: B, f: (k: K, a: A, b: B) => B) => (r: Readonly<Record<K, A>>) => B
/**
 * @since 3.0.0
 */
export declare const traverseWithIndex: (
  O: Ord<string>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <K extends string, A, S, R, W, E, B>(
  f: (k: K, a: A) => Kind<F, S, R, W, E, B>
) => (r: Readonly<Record<K, A>>) => Kind<F, S, R, W, E, Readonly<Record<K, B>>>
/**
 * @since 3.0.0
 */
export declare function traverse(
  O: Ord<string>
): <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<F, S, R, W, E, B>
) => <K extends string>(ta: ReadonlyRecord<K, A>) => Kind<F, S, R, W, E, ReadonlyRecord<K, B>>
/**
 * @since 3.0.0
 */
export declare const sequence: (
  O: Ord<string>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <K extends string, S, R, W, E, A>(
  ta: Readonly<Record<K, Kind<F, S, R, W, E, A>>>
) => Kind<F, S, R, W, E, Readonly<Record<K, A>>>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function partitionMapWithIndex<K extends string, A, B, C>(
  f: (key: K, a: A) => Either<B, C>
): (r: ReadonlyRecord<K, A>) => readonly [ReadonlyRecord<string, B>, ReadonlyRecord<string, C>]
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare function filterMapWithIndex<K extends string, A, B>(
  f: (key: K, a: A) => Option<B>
): (r: ReadonlyRecord<K, A>) => ReadonlyRecord<string, B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(
  f: (a: A) => option.Option<B>
) => (fa: ReadonlyRecord<string, A>) => ReadonlyRecord<string, B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => (fa: Readonly<Record<string, A>>) => readonly [Readonly<Record<string, B>>, Readonly<Record<string, C>>]
/**
 * @since 3.0.0
 */
export declare const reduce: (
  O: Ord<string>
) => <B, A>(b: B, f: (b: B, a: A) => B) => (r: Readonly<Record<string, A>>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: (
  O: Ord<string>
) => <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (r: Readonly<Record<string, A>>) => M
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: (
  O: Ord<string>
) => <B, A>(b: B, f: (a: A, b: B) => B) => (r: Readonly<Record<string, A>>) => B
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <A>(r: Readonly<Record<string, option.Option<A>>>) => Readonly<Record<string, A>>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <A, B>(
  r: Readonly<Record<string, Either<A, B>>>
) => readonly [Readonly<Record<string, A>>, Readonly<Record<string, B>>]
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface ReadonlyRecordλ extends TypeLambda {
  readonly type: ReadonlyRecord<string, this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: (O: Ord<string>) => <A>(S: Show<A>) => Show<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare function getEq<A, K extends string>(E: Eq<A>): Eq<ReadonlyRecord<K, A>>
/**
 * Returns a `Monoid` instance for `ReadonlyRecord`s given a `Semigroup` instance for their values.
 *
 * @example
 * import * as N from 'fp-ts/number'
 * import { getMonoid } from 'fp-ts/ReadonlyRecord'
 * import { pipe } from 'fp-ts/function'
 *
 * const M = getMonoid(N.SemigroupSum)
 * assert.deepStrictEqual(pipe({ foo: 123 }, M.combine({ foo: 456 })), { foo: 579 })
 *
 * @category instances
 * @since 3.0.0
 */
export declare function getMonoid<A, K extends string>(S: Semigroup<A>): Monoid<ReadonlyRecord<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<ReadonlyRecordλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Readonly<Record<string, (a: A) => B>>) => Readonly<Record<string, B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FunctorWithIndex: functorWithIndex.FunctorWithIndex<ReadonlyRecordλ, string>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldable: (O: Ord<string>) => Foldable<ReadonlyRecordλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldableWithIndex: (
  O: Ord<string>
) => foldableWithIndex.FoldableWithIndex<ReadonlyRecordλ, string>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: compactable.Compactable<ReadonlyRecordλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: filterable.Filterable<ReadonlyRecordλ>
/**
 * @since 3.0.0
 */
export declare const filter: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): (
    fc: ReadonlyRecord<string, C>
  ) => ReadonlyRecord<string, B>
  <B extends A, A = B>(predicate: Predicate<A>): (fb: ReadonlyRecord<string, B>) => ReadonlyRecord<string, B>
}
/**
 * @since 3.0.0
 */
export declare const partition: {
  <C extends A, B extends A, A = C>(refinement: Refinement<A, B>): (
    fc: Readonly<Record<string, C>>
  ) => readonly [Readonly<Record<string, C>>, Readonly<Record<string, B>>]
  <B extends A, A = B>(predicate: Predicate<A>): (
    fb: Readonly<Record<string, B>>
  ) => readonly [Readonly<Record<string, B>>, Readonly<Record<string, B>>]
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FilterableWithIndex: filterableWithIndex.FilterableWithIndex<ReadonlyRecordλ, string>
/**
 * @since 3.0.0
 */
export declare const filterWithIndex: {
  <C extends A, B extends A, A = C>(refinement: (i: string, a: A) => a is B): (
    fc: Readonly<Record<string, C>>
  ) => Readonly<Record<string, B>>
  <B extends A, A = B>(predicate: (i: string, a: A) => boolean): (
    fb: Readonly<Record<string, B>>
  ) => Readonly<Record<string, B>>
}
/**
 * @since 3.0.0
 */
export declare const partitionWithIndex: {
  <C extends A, B extends A, A = C>(refinement: (i: string, a: A) => a is B): (
    fb: Readonly<Record<string, C>>
  ) => readonly [Readonly<Record<string, C>>, Readonly<Record<string, B>>]
  <B extends A, A = B>(predicate: (i: string, a: A) => boolean): (
    fb: Readonly<Record<string, B>>
  ) => readonly [Readonly<Record<string, B>>, Readonly<Record<string, B>>]
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversable: (O: Ord<string>) => traversable.Traversable<ReadonlyRecordλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversableWithIndex: (
  O: Ord<string>
) => traversableWithIndex.TraversableWithIndex<ReadonlyRecordλ, string>
/**
 * @category FilterableWithEffect
 * @since 3.0.0
 */
export declare const getFilterMapE: (
  O: Ord<string>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<F, S, R, W, E, option.Option<B>>
) => (ta: Readonly<Record<string, A>>) => Kind<F, S, R, W, E, Readonly<Record<string, B>>>
/**
 * @category FilterableWithEffect
 * @since 3.0.0
 */
export declare const getPartitionMapE: (
  O: Ord<string>
) => <F extends TypeLambda>(
  F: Applicative<F>
) => <A, S, R, W, E, B, C>(
  f: (a: A) => Kind<F, S, R, W, E, Either<B, C>>
) => (
  wa: Readonly<Record<string, A>>
) => Kind<F, S, R, W, E, readonly [Readonly<Record<string, B>>, Readonly<Record<string, C>>]>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFilterableWithEffect: (
  O: Ord<string>
) => filterableWithEffect.FilterableWithEffect<ReadonlyRecordλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionSemigroup: <A>(S: Semigroup<A>) => Semigroup<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionMonoid: <A>(S: Semigroup<A>) => Monoid<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getIntersectionSemigroup: <A>(S: Semigroup<A>) => Semigroup<Readonly<Record<string, A>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getDifferenceMagma: <A>() => Magma<Readonly<Record<string, A>>>
/**
 * Calculate the number of key/value pairs in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const size: <A>(r: Readonly<Record<string, A>>) => number
/**
 * Test whether a `ReadonlyRecord` is empty.
 *
 * @since 3.0.0
 */
export declare const isEmpty: <A>(r: Readonly<Record<string, A>>) => r is Readonly<Record<string, never>>
/**
 * Map a `ReadonlyRecord` into an `ReadonlyArray`.
 *
 * @example
 * import { collect } from 'fp-ts/ReadonlyRecord'
 * import * as S from 'fp-ts/string'
 *
 * const x: { a: string, b: boolean } = { a: 'foo', b: false }
 * assert.deepStrictEqual(
 *   collect(S.Ord)((key, val) => ({key: key, value: val}))(x),
 *   [{key: 'a', value: 'foo'}, {key: 'b', value: false}]
 * )
 *
 * @since 3.0.0
 */
export declare const collect: (
  O: Ord<string>
) => <K extends string, A, B>(f: (k: K, a: A) => B) => (r: Readonly<Record<K, A>>) => readonly B[]
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const toReadonlyArray: (
  O: Ord<string>
) => <K extends string, A>(r: Readonly<Record<K, A>>) => readonly (readonly [K, A])[]
/**
 * Unfolds a `ReadonlyRecord` into a data structure of key/value pairs.
 *
 * @since 3.0.0
 */
export declare function toUnfoldable(
  O: Ord<string>
): <F extends TypeLambda>(
  U: Unfoldable<F>
) => <K extends string, A, S, R, W, E>(r: ReadonlyRecord<K, A>) => Kind<F, S, R, W, E, readonly [K, A]>
/**
 * Test whether or not a key exists in a `ReadonlyRecord`.
 *
 * Note. This function is not pipeable because is a custom type guard.
 *
 * @since 3.0.0
 */
export declare const has: <K extends string>(k: string, r: Readonly<Record<K, unknown>>) => k is K
/**
 * Test whether one `ReadonlyRecord` contains all of the keys and values contained in another `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const isSubrecord: <A>(
  E: eq.Eq<A>
) => (second: Readonly<Record<string, A>>) => (self: Readonly<Record<string, A>>) => boolean
/**
 * Lookup the value for a key in a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const lookup: (k: string) => <A>(r: Readonly<Record<string, A>>) => option.Option<A>
/**
 * An empty `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const empty: ReadonlyRecord<string, never>
/**
 * @since 3.0.0
 */
export declare function every<A, B extends A>(
  refinement: Refinement<A, B>
): Refinement<ReadonlyRecord<string, A>, ReadonlyRecord<string, B>>
export declare function every<A>(predicate: Predicate<A>): Predicate<ReadonlyRecord<string, A>>
/**
 * @since 3.0.0
 */
export declare const some: <A>(predicate: (a: A) => boolean) => (r: Readonly<Record<string, A>>) => boolean
/**
 * Tests whether a value is a member of a `ReadonlyRecord`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: eq.Eq<A>) => (a: A) => (r: Readonly<Record<string, A>>) => boolean
/**
 * @since 3.0.0
 */
export declare const union: <A>(
  M: Magma<A>
) => (second: Readonly<Record<string, A>>) => (self: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * @since 3.0.0
 */
export declare const intersection: <A>(
  M: Magma<A>
) => (second: Readonly<Record<string, A>>) => (self: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * @since 3.0.0
 */
export declare const difference: <A>(
  second: Readonly<Record<string, A>>
) => (self: Readonly<Record<string, A>>) => Readonly<Record<string, A>>
/**
 * Converts a `ReadonlyRecord` into a `ReadonlyArray` of `[key, value]` tuples.
 *
 * @example
 * import { toEntries } from 'fp-ts/ReadonlyRecord'
 *
 * assert.deepStrictEqual(toEntries({ a: 1, b: 2 }), [['a', 1], ['b', 2]])
 *
 * @since 3.0.0
 */
export declare const toEntries: <K extends string, A>(r: Readonly<Record<K, A>>) => readonly (readonly [K, A])[]
/**
 * Converts a `ReadonlyArray` of `[key, value]` tuples into a `ReadonlyRecord`.
 *
 * @example
 * import { fromEntries } from 'fp-ts/ReadonlyRecord'
 *
 * assert.deepStrictEqual(fromEntries([['a', 1], ['b', 2], ['a', 3]]), { b: 2, a: 3 })
 *
 * @since 3.0.0
 */
export declare const fromEntries: <A>(fa: readonly (readonly [string, A])[]) => Readonly<Record<string, A>>
