/**
 * @since 3.0.0
 */
import type { Either } from './Either'
import * as eq from './Eq'
import type { Magma } from './Magma'
import type { Monoid } from './Monoid'
import type { Option } from './Option'
import type { Ord } from './Ord'
import * as predicate from './Predicate'
import type { Refinement } from './Refinement'
import type { Semigroup } from './Semigroup'
import type { Show } from './Show'
import type { Eq } from './Eq'
import type { Predicate } from './Predicate'
/**
 * Create a `ReadonlySet` from one element.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const singleton: <A>(a: A) => ReadonlySet<A>
/**
 * Create a `ReadonlySet` from a `ReadonlyArray`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const fromReadonlyArray: <A>(E: eq.Eq<A>) => (as: readonly A[]) => ReadonlySet<A>
/**
 * Checks an element is a member of a set;
 * If yes, removes the value from the set
 * If no, inserts the value to the set
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const toggle: <A>(E: eq.Eq<A>) => (a: A) => (set: ReadonlySet<A>) => ReadonlySet<A>
/**
 * Return the union of two `ReadonlySet`s.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const union: <A>(E: eq.Eq<A>) => (that: ReadonlySet<A>) => (self: ReadonlySet<A>) => ReadonlySet<A>
/**
 * The `ReadonlySet` of elements which are in both the first and second `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const intersection: <A>(
  E: eq.Eq<A>
) => (that: ReadonlySet<A>) => (self: ReadonlySet<A>) => ReadonlySet<A>
/**
 * Return the set difference (`x` - `y`).
 *
 * @example
 * import { difference } from 'fp-ts/ReadonlySet'
 * import * as N from 'fp-ts/number'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(new Set([1, 2]), difference(N.Eq)(new Set([1, 3]))), new Set([2]))
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const difference: <A>(E: eq.Eq<A>) => (that: ReadonlySet<A>) => (self: ReadonlySet<A>) => ReadonlySet<A>
/**
 * Insert a value into a `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const insert: <A>(E: eq.Eq<A>) => (a: A) => (s: ReadonlySet<A>) => ReadonlySet<A>
/**
 * Delete a value from a `ReadonlySet`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const remove: <A>(E: eq.Eq<A>) => (a: A) => (s: ReadonlySet<A>) => ReadonlySet<A>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <B>(E: eq.Eq<B>) => <A>(f: (x: A) => B) => (s: ReadonlySet<A>) => ReadonlySet<B>
/**
 * @category Flattenable
 * @since 3.0.0
 */
export declare const flatMap: <B>(
  E: eq.Eq<B>
) => <A>(f: (x: A) => ReadonlySet<B>) => (s: ReadonlySet<A>) => ReadonlySet<B>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <A>(E: eq.Eq<A>) => (fa: ReadonlySet<Option<A>>) => ReadonlySet<A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <E, A>(
  EE: eq.Eq<E>,
  EA: eq.Eq<A>
) => (fa: ReadonlySet<Either<E, A>>) => readonly [ReadonlySet<E>, ReadonlySet<A>]
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare function filter<A, B extends A>(refinement: Refinement<A, B>): (s: ReadonlySet<A>) => ReadonlySet<B>
export declare function filter<A>(predicate: Predicate<A>): <B extends A>(s: ReadonlySet<B>) => ReadonlySet<B>
export declare function filter<A>(predicate: Predicate<A>): (s: ReadonlySet<A>) => ReadonlySet<A>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <B>(
  E: eq.Eq<B>
) => <A>(f: (a: A) => Option<B>) => (fa: ReadonlySet<A>) => ReadonlySet<B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare function partition<A, B extends A>(
  refinement: Refinement<A, B>
): (s: ReadonlySet<A>) => readonly [ReadonlySet<A>, ReadonlySet<B>]
export declare function partition<A>(
  predicate: Predicate<A>
): <B extends A>(s: ReadonlySet<B>) => readonly [ReadonlySet<B>, ReadonlySet<B>]
export declare function partition<A>(
  predicate: Predicate<A>
): (s: ReadonlySet<A>) => readonly [ReadonlySet<A>, ReadonlySet<A>]
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <B, C>(
  EB: eq.Eq<B>,
  EC: eq.Eq<C>
) => <A>(f: (a: A) => Either<B, C>) => (s: ReadonlySet<A>) => readonly [ReadonlySet<B>, ReadonlySet<C>]
/**
 * @category instances
 * @since 3.0.0
 */
export declare type URI = 'ReadonlySet'
declare module './HKT' {
  interface URItoKind<A> {
    readonly ReadonlySet: ReadonlySet<A>
  }
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<ReadonlySet<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: eq.Eq<A>) => eq.Eq<ReadonlySet<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionSemigroup: <A>(E: eq.Eq<A>) => Semigroup<ReadonlySet<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionMonoid: <A>(E: eq.Eq<A>) => Monoid<ReadonlySet<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getIntersectionSemigroup: <A>(E: eq.Eq<A>) => Semigroup<ReadonlySet<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getDifferenceMagma: <A>(E: eq.Eq<A>) => Magma<ReadonlySet<A>>
/**
 * An empty `ReadonlySet`.
 *
 * @since 3.0.0
 */
export declare const empty: ReadonlySet<never>
/**
 * Test whether a `ReadonlySet` is empty.
 *
 * @since 3.0.0
 */
export declare const isEmpty: <A>(set: ReadonlySet<A>) => boolean
/**
 * Calculate the number of elements in a `ReadonlySet`.
 *
 * @since 3.0.0
 */
export declare const size: <A>(set: ReadonlySet<A>) => number
/**
 * @since 3.0.0
 */
export declare const some: <A>(predicate: predicate.Predicate<A>) => (s: ReadonlySet<A>) => boolean
/**
 * @since 3.0.0
 */
export declare function every<A, B extends A>(r: Refinement<A, B>): Refinement<ReadonlySet<A>, ReadonlySet<B>>
export declare function every<A>(p: Predicate<A>): Predicate<ReadonlySet<A>>
/**
 * Return `true` if and only if every element in the first `ReadonlySet` is an element of the second `ReadonlySet`.
 *
 * @since 3.0.0
 */
export declare const isSubset: <A>(E: eq.Eq<A>) => (second: ReadonlySet<A>) => (self: ReadonlySet<A>) => boolean
/**
 * Tests whether a value is a member of a `ReadonlySet`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: eq.Eq<A>) => (a: A) => (s: ReadonlySet<A>) => boolean
/**
 * @since 3.0.0
 */
export declare const reduce: <A>(O: Ord<A>) => <B>(b: B, f: (b: B, a: A) => B) => (fa: ReadonlySet<A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: <A>(O: Ord<A>) => <M>(M: Monoid<M>) => (f: (a: A) => M) => (fa: ReadonlySet<A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRight: <A>(O: Ord<A>) => <B>(b: B, f: (a: A, b: B) => B) => (fa: ReadonlySet<A>) => B
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlySet`.
 *
 * @since 3.0.0
 */
export declare const toReadonlyArray: <A>(O: Ord<A>) => (s: ReadonlySet<A>) => readonly A[]
