/**
 * @since 3.0.0
 */
import type * as applicative from './Applicative'
import * as apply from './Apply'
import * as flattenable from './Flattenable'
import type { Endomorphism } from './Endomorphism'
import type { FromState as FromState_ } from './FromState'
import * as functor from './Functor'
import type { TypeLambda } from './HKT'
import type * as monad from './Monad'
import type * as pointed from './Pointed'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface State<S, A> {
  (s: S): readonly [S, A]
}
/**
 * Get the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const get: <S>() => State<S, S>
/**
 * Set the state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const put: <S>(s: S) => State<S, void>
/**
 * Modify the state by applying a function to the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const modify: <S>(f: Endomorphism<S>) => State<S, void>
/**
 * Get a value which depends on the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const gets: <S, A>(f: (s: S) => A) => State<S, A>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <S>(self: State<S, A>) => State<S, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <S, A>(fa: State<S, A>) => <B>(self: State<S, (a: A) => B>) => State<S, B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A, S>(a: A) => State<S, A>
/**
 * @since 3.0.0
 */
export declare const unit: <S>() => State<S, void>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Flattenable
 * @since 3.0.0
 */
export declare const flatMap: <A, S, B>(f: (a: A) => State<S, B>) => (self: State<S, A>) => State<S, B>
/**
 * Derivable from `Flattenable`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <S, A>(mma: State<S, State<S, A>>) => State<S, A>
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Stateλ extends TypeLambda {
  readonly type: State<this['InOut1'], this['Out1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Stateλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <S, B>(fab: State<S, (a: A) => B>) => State<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: pointed.Pointed<Stateλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: apply.Apply<Stateλ>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipLeftPar: <S, B>(second: State<S, B>) => <A>(self: State<S, A>) => State<S, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const zipRightPar: <S, B>(second: State<S, B>) => <A>(self: State<S, A>) => State<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: applicative.Applicative<Stateλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Flattenable: flattenable.Flattenable<Stateλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: monad.Monad<Stateλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromState: FromState_<Stateλ>
/**
 * Run a computation in the `State` monad, discarding the final state.
 *
 * @since 3.0.0
 */
export declare const evaluate: <S>(s: S) => <A>(ma: State<S, A>) => A
/**
 * Run a computation in the `State` monad discarding the result.
 *
 * @since 3.0.0
 */
export declare const execute: <S>(s: S) => <A>(ma: State<S, A>) => S
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <S, A>(
  fa: State<S, A>
) => State<
  S,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <S>(fa: State<S, A>) => State<
  S,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, S, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => State<S, B>
) => (ma: State<S, A>) => State<
  S,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const bindPar: <N extends string, A, S, B>(
  name: Exclude<N, keyof A>,
  fb: State<S, B>
) => (fa: State<S, A>) => State<
  S,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const tupled: <S, A>(fa: State<S, A>) => State<S, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <S, B>(
  fb: State<S, B>
) => <A extends ReadonlyArray<unknown>>(fas: State<S, A>) => State<S, readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, S, B>(
  f: (index: number, a: A) => State<S, B>
) => (as: readonly [A, ...A[]]) => State<S, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, S, B>(
  f: (index: number, a: A) => State<S, B>
) => (as: readonly A[]) => State<S, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, S, B>(
  f: (a: A) => State<S, B>
) => (as: readonly [A, ...A[]]) => State<S, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, S, B>(
  f: (a: A) => State<S, B>
) => (as: readonly A[]) => State<S, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <S, A>(arr: ReadonlyArray<State<S, A>>) => State<S, ReadonlyArray<A>>
