/**
 * @since 3.0.0
 */
import type * as applicative from './Applicative'
import type { Apply } from './Apply'
import type * as bifunctor from './Bifunctor'
import type { Flattenable } from './Flattenable'
import type { FlattenableRec } from './FlattenableRec'
import type * as comonad from './Comonad'
import type * as foldable from './Foldable'
import * as functor from './Functor'
import type { TypeLambda, Kind } from './HKT'
import type { Monad } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed } from './Pointed'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Semigroup } from './Semigroup'
import type * as composable from './Composable'
import type * as traversable from './Traversable'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Writer<W, A> = readonly [W, A]
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface Writerλ extends TypeLambda {
  readonly type: Writer<this['Out2'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface WriterλComposable extends TypeLambda {
  readonly type: Writer<this['In1'], this['Out1']>
}
/**
 * @category type lambdas
 * @since 3.0.0
 */
export interface WriterλFix<W> extends TypeLambda {
  readonly type: Writer<W, this['Out1']>
}
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <W>(w: W) => Writer<W, void>
/**
 * @since 3.0.0
 */
export declare const fst: <W, A>(self: Writer<W, A>) => W
/**
 * @since 3.0.0
 */
export declare const snd: <W, A>(self: Writer<W, A>) => A
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export declare const evaluate: <W, A>(self: Writer<W, A>) => A
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export declare const execute: <W, A>(self: Writer<W, A>) => W
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <W, A>(self: Writer<W, A>) => Writer<A, W>
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const listen: <W, A>(self: Writer<W, A>) => Writer<W, readonly [W, A]>
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pass: <W, A>(self: Writer<W, readonly [A, (w: W) => W]>) => Writer<W, A>
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const listens: <W, B>(f: (w: W) => B) => <A>(self: Writer<W, A>) => Writer<W, readonly [A, B]>
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const censor: <W>(f: (w: W) => W) => <A>(self: Writer<W, A>) => Writer<W, A>
/**
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <W>(self: Writer<W, A>) => Writer<W, B>
/**
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapLeft: <W, X>(f: (w: W) => X) => <A>(self: Writer<W, A>) => Writer<X, A>
/**
 * @category Bifunctor
 * @since 3.0.0
 */
export declare const mapBoth: <W, X, A, B>(f: (w: W) => X, g: (a: A) => B) => (self: Writer<W, A>) => Writer<X, B>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const compose: <B, C>(bc: Writer<B, C>) => <A>(ab: Writer<A, B>) => Writer<A, C>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const extend: <W, A, B>(f: (self: Writer<W, A>) => B) => (self: Writer<W, A>) => Writer<W, B>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const extract: <W, A>(self: Writer<W, A>) => A
/**
 * Derivable from `Extendable`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const duplicate: <W, A>(self: Writer<W, A>) => Writer<W, Writer<W, A>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const reduce: <B, A>(b: B, f: (b: B, a: A) => B) => <W>(self: Writer<W, A>) => B
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const foldMap: <M>(_M: Monoid<M>) => <A>(f: (a: A) => M) => <W>(self: Writer<W, A>) => M
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const reduceRight: <B, A>(b: B, f: (a: A, b: B) => B) => <W>(self: Writer<W, A>) => B
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const traverse: <F extends TypeLambda>(
  F: Apply<F>
) => <A, S, R, FW, E, B>(
  f: (a: A) => Kind<F, S, R, FW, E, B>
) => <W>(self: Writer<W, A>) => Kind<F, S, R, FW, E, Writer<W, B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: bifunctor.Bifunctor<Writerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: functor.Functor<Writerλ>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <W, B>(fab: Writer<W, (a: A) => B>) => Writer<W, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Composable: composable.Composable<WriterλComposable>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: comonad.Comonad<Writerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: foldable.Foldable<Writerλ>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: traversable.Traversable<Writerλ>
/**
 * @since 3.0.0
 */
export declare const sequence: <F extends TypeLambda>(
  F: Apply<F>
) => <W, FS, FR, FW, FE, A>(self: Writer<W, Kind<F, FS, FR, FW, FE, A>>) => Kind<F, FS, FR, FW, FE, Writer<W, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getPointed: <W>(M: Monoid<W>) => Pointed<WriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <W>(S: Semigroup<W>) => Apply<WriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <W>(M: Monoid<W>) => applicative.Applicative<WriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFlattenable: <W>(S: Semigroup<W>) => Flattenable<WriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <W>(M: Monoid<W>) => Monad<WriterλFix<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare function getFlattenableRec<W>(M: Monoid<W>): FlattenableRec<WriterλFix<W>>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <W>(
  S: Semigroup<W>
) => <A, B>(f: (index: number, a: A) => Writer<W, B>) => (as: readonly [A, ...A[]]) => Writer<W, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <W>(
  M: Monoid<W>
) => <A, B>(f: (index: number, a: A) => Writer<W, B>) => (as: readonly A[]) => Writer<W, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <W>(
  S: Semigroup<W>
) => <A, B>(f: (a: A) => Writer<W, B>) => (as: readonly [A, ...A[]]) => Writer<W, readonly [B, ...B[]]>
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <W>(
  M: Monoid<W>
) => <A, B>(f: (a: A) => Writer<W, B>) => (as: readonly A[]) => Writer<W, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <W>(
  M: Monoid<W>
) => <A>(arr: readonly Writer<W, A>[]) => Writer<W, readonly A[]>
