/**
 * @since 3.0.0
 */
import type { LazyArg } from './function'
import type { Monoid } from './Monoid'
import type { Semigroup } from './Semigroup'
import * as eq from './Eq'
import type * as booleanAlgebra from './BooleanAlgebra'
import type * as ord from './Ord'
import type * as show_ from './Show'
import type { Refinement } from './Refinement'
/**
 * @category refinements
 * @since 3.0.0
 */
export declare const isBoolean: Refinement<unknown, boolean>
/**
 * Defines the match over a boolean value.
 * Takes two thunks `onTrue`, `onFalse` and a `boolean` value.
 * If `value` is `false`, `onFalse()` is returned, otherwise `onTrue()`.
 *
 * @example
 * import { some, map } from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 * import { match } from 'fp-ts/boolean'
 *
 * assert.deepStrictEqual(
 *  pipe(
 *    some(true),
 *    map(match(() => 'false', () => 'true'))
 *  ),
 *  some('true')
 * )
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <A, B = A>(onFalse: LazyArg<A>, onTrue: LazyArg<B>) => (value: boolean) => A | B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Eq: eq.Eq<boolean>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const BooleanAlgebra: booleanAlgebra.BooleanAlgebra<boolean>
/**
 * `boolean` semigroup under conjunction.
 *
 * @example
 * import { SemigroupAll } from 'fp-ts/boolean'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(true, SemigroupAll.combine(true)), true)
 * assert.deepStrictEqual(pipe(true, SemigroupAll.combine(false)), false)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupAll: Semigroup<boolean>
/**
 * `boolean` semigroup under disjunction.
 *
 * @example
 * import { SemigroupAny } from 'fp-ts/boolean'
 * import { pipe } from 'fp-ts/function'
 *
 * assert.deepStrictEqual(pipe(true, SemigroupAny.combine(true)), true)
 * assert.deepStrictEqual(pipe(true, SemigroupAny.combine(false)), true)
 * assert.deepStrictEqual(pipe(false, SemigroupAny.combine(false)), false)
 *
 * @category instances
 * @since 3.0.0
 */
export declare const SemigroupAny: Semigroup<boolean>
/**
 * `boolean` monoid under conjunction.
 *
 * The `empty` value is `true`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidAll: Monoid<boolean>
/**
 * `boolean` monoid under disjunction.
 *
 * The `empty` value is `false`.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const MonoidAny: Monoid<boolean>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Ord: ord.Ord<boolean>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Show: show_.Show<boolean>
